"""重力梯度力矩分析"""
import numpy as np
from .utils import lib
from .utils import constants


def gen_sphere_unit_vector_list()->list:
    """生成球面单位矢量列表"""
    n = []
    for lat in range(-90, 90, 5):
        for lon in range(0, 360, 5):
            n.append(lib.sphere_unit_vector(np.deg2rad(lat), np.deg2rad(lon)))

    return n


def calc_gravity_gradient_torque(inertia: np.ndarray, r: float, n: list) -> np.ndarray:
    """
    计算重力梯度力矩。
    inertia: 本体系惯量矩阵
    r: 质心到地心的距离
    n: 引力方向在本体系的表示（单位矢量）
    """

    # 确保 I 是 3x3 矩阵
    assert inertia.shape == (3, 3), "I must be a 3x3 matrix"

    # 单位化
    n = lib.unit_vector(n)

    # 计算重力梯度力矩
    torque = (3 * constants.GM_EARTH / r**3) * np.cross(n, inertia@n)

    return torque


def get_max_torque(inertia, r: float) -> float:
    """
    计算重力梯度力矩最大值。
    inertia: 本体系惯量矩阵
    r: 质心到地心的距离
    """
    n_list = gen_sphere_unit_vector_list()
    torque_list = []
    for n in n_list:
        torque_list.append(calc_gravity_gradient_torque(inertia, r, n))

    max_torque = max(torque_list, key=np.linalg.norm)

    return np.linalg.norm(max_torque)


# 用于测试
# 在 src 目录下执行以下命令
# py -m aocs_lab.gravity_gradient_torque
if __name__ == '__main__':
    I = np.array([[89, 19, -7],
                  [19, 341, 2],
                  [-7, 2, 294]])

    print(f"最大重力梯度力矩为: {get_max_torque(I, 6900e3):.1e} Nm")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
pyIGRF: code to synthesise magnetic field values from any generation of the
        International Geomagnetic Reference Field (IGRF),

 @author: Ciaran Beggan (British Geological Survey)
 
 See https://www.ngdc.noaa.gov/IAGA/vmod/ for information on the IGRF
 
 Based on existing codes: igrf13.f (FORTRAN) and chaosmagpy (Python3)
 
 With acknowledgements to: Clemens Kloss (DTU Space), David Kerridge (BGS),
      William Brown and Grace Cox.
 
     This is a program for synthesising geomagnetic field values from the 
     International Geomagnetic Reference Field series of models 
     
    
     The main-field models for 1900.0, 1905.0,..1940.0 and 2025.0 are 
     non-definitive, those for 1945.0, 1950.0,...2020.0 are definitive and
     the secular-variation model for 2025.0 to 2030.0 is non-definitive.

     Main-field models are to degree and order 10 (i.e. 120 coefficients)
     for 1900.0-1995.0 and to 13 (i.e. 195 coefficients) for 2000.0 onwards. 
     The predictive secular-variation model is to degree and order 8 (i.e. 80
     coefficients).

     Older versions of IGRF from generation 1 to 12 can be used. These have 
     differing spatial resolution. 

 Inputs:
 -------
     Inputs are via the command line:    
     
     Enter IGRF version or generation: 
	default IGRF-14

     Options include: 
         Write to (1) screen or (2) filename          
    
    Load in a particular generation of IGRF v1 to v14
    
    Type of computation:
         (1) values at a single locations at one time (spot value)
         (2) values at same location at one year intervals (time series), 
         (3) grid of values at one time (grid); 
     
        Positions can be in:  
         (1) geodetic (WGS-84)
         (2) geocentric coordinates
         
     Latitude & longitude entered as: 
         (1) decimal degrees or 
         (2) degrees & minutes (not in grid)
    
     Altitude can be entered as:
         (1) Geodetic: height above the ellipsoid in km
         (2) Geocentric: distance from the Earth's centre in km
        
     Date: in decimal years (e.g. 2020.25)
 
 Outputs: 
 -----------
      : main field (in nanoTesla) and secular variation (in nanoTesla/year)
 
 
 Dependencies: 
 -------------
     : numpy, scipy
 
 Recent history of code:
 -----------------------
     Initial release: April 2020 (Ciaran Beggan, BGS)
     Corrections in 2020/2021
     Read in IGRFX.SHC files: Jun 2024
     Added IGRF14.SHC file: Nov 2024
 
    
"""
from scipy import interpolate
import igrf_utils as iut
import io_options as ioo



if __name__ == '__main__':
    # Introduction text and initial option selection
    # -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
    
    print(' ')
    print('******************************************************')
    print('*              IGRF SYNTHESIS PROGRAM                *')
    print('*                                                    *')
    print('* A program for the computation of geomagnetic       *')
    print('* field elements from the International Geomagnetic  *')
    print('* Reference Field (14th generation) as revised in    *')
    print('* December 2024 by the IAGA Working Group V-MOD.     *')
    print('*                                                    *')
    print('* It is valid for dates from 1900.0 to 2030.0;       *')
    print('* values up to 2035.0 will be computed with          *')
    print('* reduced accuracy. Values for dates before 1945.0   *')
    print('* and after 2020.0 are non-definitive, otherwise     *')
    print('* the values are definitive.                         *')
    print('*                                                    *')
    print('*                                                    *')
    print('*            (on behalf of) IAGA Working Group V-MOD *')
    print('******************************************************')
    print(' ')
    print('Enter number of require IGRF generation (1 to 14)')
    print('or press "Return" for IGRF-14')

    igrf_gen = input("Enter generation number: ")    
    
    if not igrf_gen:
        print('Using IGRF-14 ')
        igrf_gen = '14'
    else:        
        while (int(igrf_gen) < 1) or (int(igrf_gen) > 14):
            igrf_gen = input("Enter generation number: ") 


    # Load in the file of coefficients
    IGRF_FILE = r'./SHC_files/IGRF' + igrf_gen + '.SHC'
    igrf = iut.load_shcfile(IGRF_FILE, None)
    
    
    print('Enter name of output file')
    print('or press "Return" for output to screen')
      
    name = input("Enter filename: ")
    
    if not name:         # is empty 
        print('Printing to screen')
    else:
        print(name)
     
    
    while 1:
        print( 'Choose an option:')
        print( '1 - values at one location and date')
        print( '2 - values at yearly intervals at one location')
        print( '3 - values on a latitude/longitude grid at one date')
        iopt = input('->')
        iopt = iut.check_int(iopt)
        if iopt < 1 or iopt > 3: continue
        else:
            break
        
    # Parse the inputs for computing the main field and SV values. 
    # Convert geodetic to geocentric coordinates if required 
    if iopt == 1:
        date, alt, lat, colat, lon, itype, sd, cd = ioo.option1()
    elif iopt ==2:
         date, alt, lat, colat, lon, itype, sd, cd = ioo.option2()
    else:
         date, alt, lat, colat, lon, itype, sd, cd = ioo.option3()
        
        
    # Interpolate the geomagnetic coefficients to the desired date(s)
    # -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
    f = interpolate.interp1d(igrf.time, igrf.coeffs, fill_value='extrapolate')
    coeffs = f(date)    
    
    # Compute the main field B_r, B_theta and B_phi value for the location(s) 
    Br, Bt, Bp = iut.synth_values(coeffs.T, alt, colat, lon,
                              igrf.parameters['nmax'])
    
    # For the SV, find the 5 year period in which the date lies and compute
    # the SV within that period. IGRF has constant SV between each 5 year period
    # We don't need to subtract 1900 but it makes it clearer:
    epoch = (date-1900)//5    
    epoch_start = epoch*5
    # Add 1900 back on plus 1 year to account for SV in nT per year (nT/yr):
    coeffs_sv = f(1900+epoch_start+1) - f(1900+epoch_start)   
    Brs, Bts, Bps = iut.synth_values(coeffs_sv.T, alt, colat, lon,
                              igrf.parameters['nmax'])
    
    # Use the main field coefficients from the start of each five epoch
    # to compute the SV for Dec, Inc, Hor and Total Field (F) 
    # [Note: these are non-linear components of X, Y and Z so treat separately]
    coeffsm = f(1900+epoch_start);
    Brm, Btm, Bpm = iut.synth_values(coeffsm.T, alt, colat, lon,
                              igrf.parameters['nmax'])
    
    
    # Rearrange to X, Y, Z components 
    X = -Bt; Y = Bp; Z = -Br
    # For the SV
    dX = -Bts; dY = Bps; dZ = -Brs 
    Xm = -Btm; Ym = Bpm; Zm = -Brm
    # Rotate back to geodetic coords if needed
    if (itype == 1):
        t = X; X = X*cd + Z*sd;  Z = Z*cd - t*sd
        t = dX; dX = dX*cd + dZ*sd;  dZ = dZ*cd - t*sd
        t = Xm; Xm = Xm*cd + Zm*sd;  Zm = Zm*cd - t*sd
        
    # Compute the four non-linear components 
    dec, hoz, inc, eff = iut.xyz2dhif(X,Y,Z)
    # The IGRF SV coefficients are relative to the main field components 
    # at the start of each five year epoch e.g. 2010, 2015, 2020
    decs, hozs, incs, effs = iut.xyz2dhif_sv(Xm, Ym, Zm, dX, dY, dZ)
    
    
    # Finally, parse the outputs for writing to screen or file
    if iopt == 1:
        ioo.write1(name, date, alt, lat, colat, lon, X, Y, Z, dX, dY, dZ, \
                  dec, hoz, inc, eff, decs, hozs, incs, effs, itype, igrf_gen)
        if name:
            print('Written to file: ' + name )
    elif iopt == 2:
        ioo.write2(name, date, alt, lat, colat, lon, X, Y, Z, dX, dY, dZ, \
                  dec, hoz, inc, eff, decs, hozs, incs, effs, itype, igrf_gen)
        if name:
            print('Written to file: ' + name )
    else:
        ioo.write3(name, date, alt, lat, colat, lon, X, Y, Z, dX, dY, dZ, \
                  dec, hoz, inc, eff, decs, hozs, incs, effs, itype, igrf_gen)
        if name:
            print('Written to file: ' + name )
    



"""飞轮惯滑时间计算"""
import numpy as np
from aocs_lab.utils import time
from aocs_lab.influxdb import influxdb2
from aocs_lab.wheel import wheel_data_process


def gen_database(sat_name: str) -> dict:
    """生成数据库信息"""
    database = {}
    database['url'] = "http://172.16.110.211:8086"
    database['token'] = (
        "u0OGUlvv6IGGYSxpoZGZNjenBtE-1ADcdun-W-0oacL66cef5DXPmDwVzj93oP1MRBBCCUOWNFS9yMb77o5OCQ=="
    )
    database['org'] = "gs"
    database['bucket'] = f'piesat02_{sat_name}_database'

    return database

# wheel_test_a02 = {
#     "start_time": '2025-02-12T15:10:00',
#     "end_time": '2025-02-12T15:30:00',
#     "tm_tag": ['TMK738', 'TMK739', 'TMK740', 'TMK741']
# }


def letters_to_numbers(letters):
    """字母转数字"""
    return [ord(letter) - ord('A') for letter in letters]


def number_to_letter(number):
    """数字转字母"""
    return chr(number + ord('A'))


def calc_slide_time(
        start_time: str,
        end_time: str,
        sat_name: str,
        wheel_list: list = None,
        tm_tag: list = None) -> str:
    """
    计算飞轮惯滑时间

    Args:
        start_time: 开始时间
        end_time: 结束时间
        sat_name: 卫星名称。例如 'b01'
        wheel_list: 飞轮列表。例如 ['A', 'B', 'C', 'D']
        tm_tag: TM标签。默认为 ['TMKA553', 'TMKA561', 'TMKA569', 'TMKA577']
    """

    if wheel_list is None:
        wheel_list = ['A', 'B', 'C', 'D']

    if tm_tag is None:
        tm_tag = ['TMKA553', 'TMKA561', 'TMKA569', 'TMKA577']

    data_list = influxdb2.get_field_value_from_influxdb(
        gen_database(sat_name),
        time.beijing_to_utc_time_str(start_time),
        time.beijing_to_utc_time_str(end_time),
        tm_tag)

    data_array = np.array(data_list)

    for i in letters_to_numbers(wheel_list):
        dt = wheel_data_process.calc_slide_time(
            data_array[:, 0], data_array[:, i+1], 5990, 3000)
        print(f"飞轮 {number_to_letter(i)} 惯滑时间为 {dt:.0f} s")
        return f"飞轮 {number_to_letter(i)} 惯滑时间为 {dt:.0f} s"


if __name__ == '__main__':
    calc_slide_time(
        '2025-02-19T16:20:00',
        '2025-02-19T16:50:00',
        'b01',
        ['C'])



"""分析刚体主轴惯量与主轴惯量坐标系"""
from scipy.spatial.transform import Rotation as R
import numpy as np
from .utils import lib


def analyse_principal_moments_of_inertia(inertia_ref):
    """
    分析主轴惯量
    inertia_ref: 参考系 ref 下惯量矩阵
    """
    # 计算特征值和特征向量
    principal_inertia, principal_axes = np.linalg.eig(inertia_ref)
    err = lib.orthogonality_error(principal_axes)

    rotvec = R.from_matrix(principal_axes).as_rotvec()
    rotvec_norm_deg = np.rad2deg(np.linalg.norm(rotvec))

    print("主轴惯量 (kg*m^2):")
    print(f'  Ixx = {principal_inertia[0]:6.1f}')
    print(f'  Iyy = {principal_inertia[1]:6.1f}')
    print(f'  Izz = {principal_inertia[2]:6.1f}')
    print("")
    print(f'主惯量轴坐标系正交性误差:\n  {err:.3e}')
    print("")
    print(f'主惯量轴坐标系相对本体系偏转角度:\n  {rotvec_norm_deg:.3f} deg')


if __name__ == '__main__':
    I_B = np.array([[89, 19, -7],
                    [19, 341, 2],
                    [-7, 2, 294]])

    analyse_principal_moments_of_inertia(I_B)



#!/usr/bin/env python3
"""
python tests/gyro/gyro_dat_plot.py --file ./tests/gyro/GyroA50HzData.dat --save ./tests/gyro/GyroA50HzData.png
"""
from __future__ import annotations

import argparse
import os
import sys
from pathlib import Path

import numpy as np
import matplotlib.pyplot as plt


def load_gyro_dat(file_path: os.PathLike | str) -> np.ndarray:
    """Load big-endian float32 triples from a .dat file into an (N, 3) array.

    Truncates trailing values if the total count is not a multiple of 3.
    """
    file_path = Path(file_path)
    if not file_path.exists():
        raise FileNotFoundError(f"File not found: {file_path}")

    # Read as big-endian float32
    raw = np.fromfile(str(file_path), dtype=np.dtype(">f4"))
    if raw.size == 0:
        raise ValueError(f"Empty or unreadable file: {file_path}")

    if (raw.size % 3) != 0:
        raise ValueError(
            f"Value count {raw.size} is not a multiple of 3 (expected groups of 3 floats)."
        )

    data = raw.reshape(-1, 3)
    return data


def plot_gyro(time_s: np.ndarray, data: np.ndarray, title: str = "Gyro Data", save: str | None = None) -> None:
    if data.shape[1] != 3:
        raise ValueError("Expected data with shape (N, 3)")
    
    # data = np.rad2deg(data)  # Convert from radians to degrees

    plt.figure(figsize=(12, 5))
    labels = ["X", "Y", "Z"]
    colors = ["tab:blue", "tab:orange", "tab:green"]

    # Plot signals and compute means
    means = []
    for i in range(3):
        plt.plot(time_s, data[:, i], label=labels[i], color=colors[i])
        m = float(np.mean(data[:, i])) if data.shape[0] > 0 else float("nan")
        means.append(m)
    # Mean lines
    for i in range(3):
        plt.hlines(means[i], xmin=time_s[0], xmax=time_s[-1], colors=colors[i], linestyles="--", alpha=1, linewidth=3)

    plt.xlabel("Time [s]")
    plt.ylabel("Value (deg/s)")
    plt.title(title)
    plt.grid(True, linestyle=":", alpha=0.6)
    plt.legend()

    # Add a statistics text box
    stats = (
        f"Means:\n"
        f"  X: {means[0]:.6f}\n"
        f"  Y: {means[1]:.6f}\n"
        f"  Z: {means[2]:.6f}"
    )
    # Place textbox at top-right inside axes
    ax = plt.gca()
    ax.text(
        0.995,
        0.98,
        stats,
        transform=ax.transAxes,
        ha="right",
        va="top",
        fontsize=9,
        bbox=dict(boxstyle="round", facecolor="white", alpha=0.75),
    )
    plt.tight_layout()

    if save:
        out = Path(save)
        out.parent.mkdir(parents=True, exist_ok=True)
        plt.savefig(out, dpi=150)
        print(f"Saved plot to: {out}")
    else:
        plt.show()


def parse_args(argv: list[str] | None = None) -> argparse.Namespace:
    p = argparse.ArgumentParser(description="Parse and plot big-endian float32 gyro .dat files (XYZ triples)")
    p.add_argument("--file", "-f", type=str, default="GyroA50HzData.dat", help="Path to .dat file (default: GyroA50HzData.dat)")
    p.add_argument("--save", type=str, default=None, help="Save plot to this path instead of showing")
    p.add_argument("--csv", type=str, default=None, help="Optionally export data to CSV (time_s,X_deg_s,Y_deg_s,Z_deg_s)")
    return p.parse_args(argv)


def main(argv: list[str] | None = None) -> int:
    args = parse_args(argv)

    dat_path = Path(args.file)
    data = load_gyro_dat(dat_path)
    title = f"Gyro Data\n{dat_path.name}"
    print(data)

    # Time slicing
    fs = 50.0  # Default sampling rate in Hz
    n = data.shape[0]
    t = np.arange(n) / float(fs)

    # Optional CSV export (degrees per second, consistent with plot)
    if args.csv:
        out = Path(args.csv)
        out.parent.mkdir(parents=True, exist_ok=True)
        # data_deg = np.rad2deg(data)
        arr = np.column_stack([t, data])
        header = "time_s,X_deg_s,Y_deg_s,Z_deg_s"
        np.savetxt(out, arr, delimiter=",", header=header, comments="")
        print(f"Saved CSV to: {out}")

    plot_gyro(t, data, title=title, save=args.save)
    return 0


if __name__ == "__main__":
    raise SystemExit(main())



"""穷举找到满足需求的星敏指向"""
import sys
import os
import time
from dataclasses import dataclass
import numpy as np
# import matplotlib.pyplot as plt
# from scipy.spatial.transform import Rotation as R


parent_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), '../src'))
sys.path.append(parent_dir)
import aocs_lab.utils.lib as lib
import aocs_lab.utils.time as time_utils



def get_angle_min(points):
    """计算点集中两两点之间的最小夹角"""
    points_num = len(points)
    start_time = time.time()

    angle_min_list = []
    for i, point_i in enumerate(points):
        angle_min = np.pi
        for j, point_j in enumerate(points):
            if i != j:
                # angle = lib.vector_angle(point_i, point_j)
                angle = np.acos(np.clip(np.dot(point_i, point_j), -1, 1))
                angle_min = min(angle, angle_min)

            if j % 1000 == 0:
                time_utils.print_percentage(i * points_num + j, points_num * points_num, start_time)

        angle_min_list.append(angle_min)

    return max(angle_min_list)


def gen_sphere_unit_vector_list() -> list:
    """生成球面单位矢量列表"""
    n = []
    for lat in range(-89, 90, 1):
        for lon in range(0, 360, 1):
            n.append(lib.sphere_unit_vector(np.deg2rad(lat), np.deg2rad(lon)))

    # 添加极点
    n.append(lib.sphere_unit_vector(np.deg2rad(-90), np.deg2rad(0)))
    n.append(lib.sphere_unit_vector(np.deg2rad(90), np.deg2rad(0)))

    return n



@dataclass
class VectorInOrbit:
    """轨道坐标系中的矢量及姿态信息"""
    sun: np.array = np.zeros(3)
    dcm_bo: np.array = np.zeros(3,3)

@dataclass
class AttMode:
    """综合一圈轨道的太阳矢量的姿态信息"""
    earth: np.array = np.array([0, 0, 1])

    position: list[VectorInOrbit] = None

    def init(self, beta):
        self.position = []
        for u in range(0, np.deg2rad(360), np.deg2rad(1)):
            sun = np.array([np.cos(np.deg2rad(beta)), np.sin(np.deg2rad(beta)), 0])
            dcm_bo = np.array([[1,0,0],[0,1,0],[0,0,1]])
            self.position.append(VectorInOrbit(sun, dcm_bo))

def get_valid_star_point(points_1, points_2, att_mode: AttMode):
    """根据定义的轨道和姿态，获取有效的星敏指向列表"""

    points_1_valid = []
    points_2_valid = []
    return points_1_valid, points_2_valid

if __name__ == "__main__":
    points = gen_sphere_unit_vector_list()

    # points_in_equ = [point for point in points if np.abs(point[2]) < 0.1]
    # angle_min = get_angle_min(points_in_equ)
    # print(f"\nangle_min: {np.rad2deg(angle_min):.2f} deg")

    print(len(gen_sphere_unit_vector_list()))


# 用经纬度遍历的方法，经纬度步长均为 1 deg，共 64800 个点
# 用螺旋法生成平均间隔 1 deg 的点，需要约 40000 个点，没有太大的改善



"""计算阳照区时间"""
import numpy as np
from .utils import lib
from .utils import constants


def sun_time(beta_angle, semimajor_axis):
    """计算阳照区时间"""
    # 地球矢量
    v_earth = np.array([0, 0, 1])

    # 太阳矢量与地球矢量夹角阈值（太阳升起）
    theta = np.arcsin(constants.EARTH_EQUATORIAL_RADIUS/semimajor_axis)

    # 圆锥矢量遍历
    earth_sun_angle = np.array([])
    phi_line = np.linspace(0, 2*np.pi, 1000)

    for phi in phi_line:
        y = np.cos(np.pi-beta_angle)
        xz = np.sin(np.pi-beta_angle)
        v_sun = np.array([xz*np.sin(phi), y, xz*np.cos(phi)])

        angle = lib.vector_angle(v_sun, v_earth)
        earth_sun_angle = np.append(earth_sun_angle, angle)

    sun_segment = np.where(earth_sun_angle >= theta)

    # 有效时间计算
    valid_segment_len = len(sun_segment[0])
    valid_segment_time = valid_segment_len / len(phi_line) * lib.orbit_period(semimajor_axis)

    print("轨道周期 (s):", lib.orbit_period(semimajor_axis))
    print("阳照区时长 (s):", valid_segment_time)


if __name__ == "__main__":
    # 太阳矢量与轨道面法线夹角
    sun_time(np.deg2rad(31), 6900e3)
    sun_time(np.deg2rad(67), 6900e3)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sat Apr  4 18:29:16 2020
@author: Ciaran Beggan (British Geological Survey)

Uses pytest

 in a terminal, run :
     
    >> py.test igrf13_tests.py
   or 
   in Spyder: In [1]: !py.test tests/igrf13_tests.py
   or 
   >> In [1]: !python -m pytest tests

pytest will find these test scripts, run them and alert you to any errors


Test values and examples for running synth_values for IGRF-13
    # Generate some test values for date, lat, lon and altitude
    date = np.arange(1900,2025, 15, dtype='int32')
    lat =  90-np.arange(-85,85, 20, dtype='int32')
    lon = np.arange(-150, 150, 30, dtype='int32')
    alt = np.arange(6300, 6750, 50, dtype='int32')
    for d, clt, ln, a in zip(date, lat, lon, alt):
        print('{},{},{},{}'.format(d,clt,ln,a))

    There are 9 tests in total.  Values have been checked independently 
    against igrf.f and D. Kerridge Jupyter Notebook implementation (2019)

"""


from scipy import interpolate
import igrf_utils as iut
import numpy as np
from numpy.testing import assert_allclose
import pytest

IGRF_FILE = r'./IGRF13.shc'
igrf = iut.load_shcfile(IGRF_FILE, None)
f = interpolate.interp1d(igrf.time, igrf.coeffs)

@pytest.mark.parametrize('date, lat, lon, alt, expected_Bxyz',                       
           [ (1900, 175, -150, 6300, np.array([-5072.93,10620.34,-67233.55]) ),
             (1915, 155, -120, 6350, np.array([14692.62,12387.97,-59640.81]) ),
             (1930, 135, -90, 6400,  np.array([23925.47,10358.94,-30640.98]) ),
             (1945, 115, -60, 6450,  np.array([23642.86, -200.29, -7607.92]) ),
             (1960, 95, -30, 6500,   np.array([23647.00,-9302.27, -3610.73]) ),
             (1975, 75, 0, 6550,     np.array([30050.59,-3367.82,  6332.69]) ),
             (1990, 55, 30, 6600,    np.array([25224.81, 1058.25, 30965.61]) ),
             (2005, 35, 60, 6650,    np.array([14718.37, 2842.99, 46050.88]) ),
             (2020, 15, 90, 6700,    np.array([ 3732.49, 1293.25, 50833.96]) )
             ])

def test_synth_values(date, lat, lon, alt, expected_Bxyz ):

        coeffs = f(date) 
        
        # Compute the Br, Btheta and Bphi value for this location 
        found = iut.synth_values(coeffs.T, alt, lat, lon,
                          igrf.parameters['nmax'])
        foundxyz = np.array([ -found[1], found[2], -found[0] ])
        assert_allclose(foundxyz, expected_Bxyz, rtol=1e-02, atol=1e-02)



import os
import sys
import numpy as np
import matplotlib.pyplot as plt
from matplotlib import rcParams
import matplotlib.dates as mdates
from datetime import timedelta, datetime
import re


parent_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), '../../src'))
sys.path.append(parent_dir)
import aocs_lab.utils.lib as lib
import aocs_lab.sun_time as sun_time
import aocs_lab.utils.constants as const



def calc_dnda(a):
    """
    :param a: 轨道半长轴
    :return: 轨道角速度变化量与轨道半长轴变化量之比
    """
    n = lib.orbit_angular_rate(a)
    return -3*n / (2*a)

def calc_du(a, da0, da_dot, du0, t):
    k = calc_dnda(a)
    du = k * (da0 * t + 0.5 * da_dot * t**2) + du0
    return du

def calc_da(da0, da_dot, t):
    """
    计算轨道半长轴变化量
    :param da0: 初始轨道半长轴变化量
    :param da_dot: 轨道半长轴变化率
    :param t: 时间
    :return: 轨道半长轴变化量
    """
    return da0 + da_dot * t

def plot_tube(a, du, da, dix, diy):
    _, ax = plt.subplots(nrows=2, ncols=1, figsize=(10, 10))  # 创建两个纵向子图

    # 第一个子图
    ax[0].plot(a * du, da, label='du', color='blue')
    ax[0].set_xlabel('a*du (m)')
    ax[0].set_ylabel('da (m)')
    ax[0].set_title('a*du vs da')
    ax[0].grid(True)
    ax[0].legend()
    ax[0].set_aspect('equal')

    # 第二个子图
    ax[1].plot(a*diy, da, color='green')
    ax[1].set_xlabel('a*diy (m)')
    ax[1].set_ylabel('da (m)')
    ax[1].set_title('a*diy vs da')
    ax[1].grid(True)
    ax[1].legend()
    ax[1].set_aspect('equal')

    # 标记x轴最大值和最小值对应的点
    x_vals = a * diy
    idx_max = np.argmax(x_vals)
    idx_min = np.argmin(x_vals)
    ax[1].scatter(x_vals[idx_max], da[idx_max], color='red', marker='o', label='x max')
    ax[1].scatter(x_vals[idx_min], da[idx_min], color='orange', marker='o', label='x min')
    # 在图中标注横纵坐标
    ax[1].annotate(f"({x_vals[idx_max]:.2f}, {da[idx_max]:.2f})",
                   (x_vals[idx_max], da[idx_max]),
                   textcoords="offset points", xytext=(10,10), ha='left', color='red')
    ax[1].annotate(f"({x_vals[idx_min]:.2f}, {da[idx_min]:.2f})",
                   (x_vals[idx_min], da[idx_min]),
                   textcoords="offset points", xytext=(10,-20), ha='left', color='orange')
    ax[1].legend()

    # 添加半径为250m的圆
    theta = np.linspace(0, 2 * np.pi, 200)
    circle_x = 250 * np.cos(theta)
    circle_y = 250 * np.sin(theta)
    ax[1].plot(circle_x, circle_y, 'r--', label='r=250m')
    ax[1].legend()

    plt.tight_layout()
    plt.show()

def test():
    # 轨道半长轴
    a = 6913e3
    # 轨道半长轴变化量
    da0 = 100
    # 轨道半长轴变化率
    da_dot = -200 / 86400
    # 时间
    t = 86400/2

    du0 = 1700 / a

    du = calc_du(a, da0, da_dot, du0, t)
    da = calc_da(da0, da_dot, t)

    k = const.EARTH_ROTATION_RATE / lib.orbit_angular_rate(a)

    dix = np.deg2rad(0.001)
    diy = du * k

    en_max = a * np.sqrt(dix**2 + diy**2)
    er_max = da
    e_max = np.sqrt(en_max**2 + er_max**2)

    print(f"da: {da:0.1f} m")
    print(f"du: {du} rad")
    print(f"a*du: {a*du:0.1f} m")
    print(f"er_max: {er_max:0.1f} m")
    print(f"en_max: {en_max:0.1f} m")
    print(f"e_max: {e_max:0.1f} m")

    t = np.arange(0, 86400, 60)
    du = calc_du(a, da0, da_dot, du0, t)
    da = calc_da(da0, da_dot, t)
    diy = du * k

    plot_tube(a, du, da, 0, diy)


if __name__ == "__main__":
    test()



# 计算飞轮惯滑时间

import numpy as np


def get_cross_index(data, threshold, cross_dir):
    index = -1

    if cross_dir == 'up':
        for i in range(len(data) - 1):
            if data[i] < threshold and data[i + 1] >= threshold:
                index = i

    elif cross_dir == 'down':
        for i in range(len(data) - 1):
            if data[i] > threshold and data[i + 1] <= threshold:
                index = i
    else:
        raise ValueError("wrong direction")

    if index < 0:
        raise ValueError("can't find cross point")

    return index


def calc_slide_time(time_list, speed_list, high_speed, low_speed):
    start_index = get_cross_index(speed_list, high_speed, 'down')
    end_index = get_cross_index(speed_list, low_speed, 'down')

    return time_list[end_index] - time_list[start_index]


if __name__ == "__main__":
    sat_info = [{"sat_name": "C01", "sat_file": "C01_wheel_speed.csv"},
                {"sat_name": "C02", "sat_file": "C02_wheel_speed.csv"},
                {"sat_name": "C03", "sat_file": "C03_wheel_speed.csv"},
                {"sat_name": "C04", "sat_file": "C04_wheel_speed.csv"}]

    for sat_index in range(len(sat_info)):
        data = np.loadtxt(sat_info[sat_index]['sat_file'],
                          delimiter=',', skiprows=1, encoding='utf-8')

        valid_index = []
        for i in range(len(data[:, 0])):
            if data[i, 1] == 0 and data[i, 2] == 0 and data[i, 3] == 0 and data[i, 4] == 0:
                pass
            else:
                valid_index.append(i)

        print(f"{sat_info[sat_index]['sat_name']} 卫星")
        for wheel_num in range(4):
            dt = calc_slide_time(
                data[valid_index, 0]/1000, data[valid_index, wheel_num + 1], 5990, 3000)
            print(f"飞轮 {wheel_num} 惯滑时间为 {dt:.0f} s")



"""文件操作相关的函数"""
import csv

def save_as_csv(csv_file_path: str, data: list[dict]):
    """保存列表数据为 CSV"""
    # 打开 CSV 文件进行写入
    with open(csv_file_path, 'w', encoding='utf-8', newline='') as csv_file:
        # 获取 CSV 的列名
        fieldnames = data[0].keys()
        writer = csv.DictWriter(csv_file, fieldnames=fieldnames)

        # 写入表头
        writer.writeheader()

        # 写入数据
        for row in data:
            writer.writerow(row)

def read_csv(csv_file_path) -> list[dict]:
    """存储CSV数据的列表"""
    data = []

    # 打开CSV文件进行读取
    with open(csv_file_path, 'r', encoding='utf-8') as file:
        reader = csv.DictReader(file)

        # 逐行读取数据
        for row in reader:
            converted_row = {key: float(value) for key, value in row.items()}
            data.append(converted_row)

    return data



from math import acos, sin, cos, gcd
import math
import os
import sys
import numpy as np
from typing import List, Tuple

parent_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), '../../src'))
sys.path.append(parent_dir)
import aocs_lab.utils.lib as lib
import aocs_lab.utils.constants as const
import aocs_lab.utils.lib as lib


J2 = 1.08263e-3
J3 = -2.565e-6
a_norm = 6378137
A2 = 1.5*J2
ns = 1.99102128e-07 # 太阳相对地球角速度
DAY_SEC = 86400.0

def repeat_orbit_search(
        height_min = 980e3, 
        height_max = 1200e3, 
        repeat_days_min = 5, 
        repeat_days_max = 25, 
        repeat_revs_min = 100, 
        repeat_revs_max = 300):
    """
    回归轨道搜索

    """

    a1 = height_min + const.EARTH_EQUATORIAL_RADIUS
    a2 = height_max + const.EARTH_EQUATORIAL_RADIUS
    e1, i1 = calc_elements(a1)
    e2, i2 = calc_elements(a2)

    T1 = calc_node_period(a1, e1, i1, np.deg2rad(90))
    T2 = calc_node_period(a2, e2, i2, np.deg2rad(90))

    p1 = T1 / DAY_SEC
    p2 = T2 / DAY_SEC

    results: List[Tuple[float, float, float, int, int]] = []

    for revs in range(repeat_revs_min, repeat_revs_max + 1):
        j_min = max(int(math.ceil(p1 * revs)), repeat_days_min)
        j_max = min(int(math.floor(p2 * revs)), repeat_days_max)
        if j_min > j_max:
            continue
        for days in range(j_min, j_max + 1):
            if gcd(revs, days) == 1:
                T = days / revs * DAY_SEC
                u = 2.0 / 3.0

                semi = (a1 / (T1 ** u) * (T2 - T) + a2 / (T2 ** u) * (T - T1)) * (T ** u) / (T2 - T1)
                e, inc = calc_elements(semi)
                altitude_km = (semi - const.EARTH_EQUATORIAL_RADIUS) / 1000.0
                inc_deg = np.rad2deg(inc)
                results.append((altitude_km, e, inc_deg, revs, days))

    return results



def calc_elements(a):
    """
    计算轨道要素
    """
    no = lib.orbit_angular_rate(a)
    a = a/a_norm
    p = a*(1 - 1e-6)
    i = acos(-ns/(A2/p**2 * no))
    e = sin(i) / (cos(i)**2/sin(i) - 2*J2*a/J3)
    
    return e, i


def calc_node_period(a,e,i,w):
    """
    计算轨道节点周期
    """
    T = lib.orbit_period(a)
    node_period = T *(1 + A2/4/(a/a_norm)**2 * ((-12 - 22 * e**2) + (16 + 29 * e**2) * sin(i)**2 + (16-20*sin(i)**2) * e * cos(w) - (12-15*sin(i)**2) * e**2 * cos(2*w)))

    return node_period

if __name__ == "__main__":
    res = repeat_orbit_search(478e3-50e3, 478e3+50e3, 0, 100, 50, 500)
    # 简单打印前若干行
    print("高度km    e        倾角deg  回归圈  回归天")
    for r in res:
        print(f"{r[0]:8.2f} {r[1]:15.5e} {r[2]:9.4f} {r[3]:7d} {r[4]:7d}")
    print(f"总组合数: {len(res)}")



import requests
import json
import matplotlib.pyplot as plt
from datetime import datetime
import pymsis


def plot_f10_7():
    """
    读取本地json文件，绘制time-tag vs predicted_f10.7曲线
    """

    with open("predicted-solar-cycle.json", "r", encoding="utf-8") as f:
        predicted_data = json.load(f)

    with open("f10-7cm-flux-smoothed.json", "r", encoding="utf-8") as f:
        historical_data = json.load(f)


    predicted_time_tags = [datetime.strptime(item["time-tag"], "%Y-%m") for item in predicted_data]
    predicted_f10_7 = [item["predicted_f10.7"] for item in predicted_data]

    historical_time_tags = [datetime.strptime(item["time-tag"], "%Y-%m") for item in historical_data]
    historical_f10_7 = [item["f10.7"] for item in historical_data]
    
    historical_smoothed_data = [item for item in historical_data if item["smoothed_f10.7"] != -1]
    historical_smoothed_time_tags = [datetime.strptime(item["time-tag"], "%Y-%m") for item in historical_smoothed_data]
    historical_smoothed_f10_7 = [item["smoothed_f10.7"] for item in historical_smoothed_data]


    # 为了避免x轴标签过于拥挤，可以每隔几个点显示一个标签，或者旋转标签
    plt.figure(figsize=(12, 6))
    plt.plot(predicted_time_tags, predicted_f10_7, label="Predicted F10.7", marker='.')
    plt.plot(historical_time_tags, historical_f10_7, label="Historical F10.7", marker='.')
    plt.plot(historical_smoothed_time_tags, historical_smoothed_f10_7, label="Historical Smoothed F10.7", marker='.')
    
    plt.xlabel("Time")
    plt.ylabel("F10.7")
    plt.title("Solar Cycle F10.7 Flux")
    plt.grid(True)
    plt.legend()
    plt.tight_layout()
    plt.show()


def F10_7():
    """
    获取F10.7太阳活动指数的示例函数

    Returns:
        f10_7: F10.7太阳活动指数
    """

    # 预测数据
    url = "https://services.swpc.noaa.gov/json/solar-cycle/predicted-solar-cycle.json"
    response = requests.get(url)
    with open("predicted-solar-cycle.json", "w", encoding="utf-8") as f:
        json.dump(response.json(), f, indent=4, ensure_ascii=False)

    # 历史数据
    url = "https://services.swpc.noaa.gov/json/solar-cycle/f10-7cm-flux-smoothed.json"
    response = requests.get(url)
    with open("f10-7cm-flux-smoothed.json", "w", encoding="utf-8") as f:
        json.dump(response.json(), f, indent=4, ensure_ascii=False)

    print("下载完成")


if __name__ == "__main__":
    # F10_7()
    # plot_f10_7()

    import numpy as np
    import pymsis

    dates = np.arange(np.datetime64("2026-10-28T00:00"), np.datetime64("2030-11-04T00:00"), np.timedelta64(30, "m"))
    # geomagnetic_activity=-1 is a storm-time run

    # aps = np.full(len(dates), 4.0)  # 或 5–7
    data = pymsis.calculate(
        dates = np.datetime64("2026-10-28T00:00"), 
        lons=0, 
        lats=0, 
        alts=400, 
        f107s=150,
        f107as=150,
        aps=4)

    
    print(data)




from mcp.server.fastmcp import FastMCP
import repeat

mcp = FastMCP()

@mcp.tool(description="获取回归轨道列表, 输入期望的轨道高度(单位米), 返回高度在该值上下50公里范围内的回归轨道列表")
def get_repeat_orbit(height: float) -> str:
    res = repeat.repeat_orbit_search(
        height - 50e3, 
        height + 50e3, 
        0, 100, 
        50, 500)

    text = "高度km    e        倾角deg  回归圈数  回归天数\n"
    for r in res:
        text = text + (f"{r[0]:8.2f} {r[1]:15.5e} {r[2]:9.4f} {r[3]:7d} {r[4]:7d}\n")

    return text



if __name__ == "__main__":
    mcp.run(transport="streamable-http")






from scipy.spatial.transform import Rotation as R

# 校正俯仰轴 (俯仰角增加为正，俯仰角减小为负)
r1 = R.from_euler('xyz', [0, -0.390555583, 0], degrees=True)

# 原始 SAR 天线安装矩阵
r2 = R.from_quat([0.9999999677867,
                  0.0001570796351,
                  -0.0000785398185,
                  -0.0001832595732], scalar_first=True)

r3 = r1*r2

q3 = r3.as_quat(scalar_first=True)

for q in q3:
    print(f'{q:.16f}')



"""just for calculate"""
from datetime import datetime, timedelta
import os
import sys
import numpy as np
import matplotlib.pyplot as plt

parent_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), '../src'))
sys.path.append(parent_dir)
import aocs_lab.influxdb.influxdb1 as influxdb1
import aocs_lab.file.csv_file as csv_file

time_info = [
    {
        "sat_name": "C01",
        "direction": "+X",
        "duration": 2400,
        # 2 Jan 2025 10:21:00.000                   21.258
        # 3 Jan 2025 10:09:00.000                   18.225
        # 4 Jan 2025 06:46:00.000                   13.912
        # 5 Jan 2025 06:33:30.000                    9.761
        # 6 Jan 2025 09:32:00.000                    7.711
        "utc": datetime.fromisoformat("2025-01-06 09:32:00.000"),
    },
    {
        "sat_name": "C02",
        "direction": "+X",
        "duration": 2400,
        # 2 Jan 2025 10:18:00.000                   25.390
        # 3 Jan 2025 10:06:00.000                   22.243
        # 5 Jan 2025 06:31:00.000                   13.396
        "utc": datetime.fromisoformat("2025-01-05 06:31:00.000"),
    },
    {
        "sat_name": "C03",
        "direction": "+X",
        "duration": 2400,
        # 6 Jan 2025 09:15:00.000                   13.442
        "utc": datetime.fromisoformat("2025-01-06 09:15:00.000"),
    },
    {
        "sat_name": "C04",
        "direction": "+X",
        "duration": 2400,
        # 14 Jan 2025 00:48:00.000                  347.274
        "utc": datetime.fromisoformat("2025-01-14 00:48:00.000"),
    },
]

evs = [
    {
        "ex": -9.1e-4,
        "ey": -1.2e-4,
        "dir": "+x",
    },
    {
        "ex": -8.6e-4,
        "ey": -2.1e-4,
        "dir": "+x",
    },
    {
        "ex": -10.7e-4,
        "ey": -2.6e-4,
        "dir": "+x",
    },
    {
        "ex": -5.3e-4,
        "ey": 1.3e-4,
        "dir": "+x",
    },
]


def anlyse_orbit():
    """分析轨道"""  
    names = piesat2c["name"]

    # ex, ey 变化
    _, ax = plt.subplots()
    for name in names:
        data = csv_file.read_csv(f"{name['sat_name']}.csv")
        ex = [e['d_ex'] for e in data]
        ey = [e['d_ey'] for e in data]
        ax.plot(ex, ey, label=name['sat_name'], marker='.')

    ax.set_aspect("equal", "box")
    ax.legend()
    ax.set_xlabel("d_ex")
    ax.set_ylabel("d_ey")
    plt.grid()
    plt.xlim(-0.002, 0.002)
    plt.ylim(-0.002, 0.002)
    plt.show()

    _, ax = plt.subplots()
    for name in names:
        data = csv_file.read_csv(f"{name['sat_name']}.csv")
        a = [e['a_mean'] for e in data]
        ax.plot(a, label=name['sat_name'], marker='.')

def anlyse_orbit2():
    """分析轨道"""
    for ev in evs:
        ex = ev["ex"]
        ey = ev["ey"]
        if ev["dir"] == "+x":
            omega = np.arctan2(-ey, -ex)
        else:
            omega = np.arctan2(ey, ex)

        if omega < 0:
            omega += 2*np.pi

        e_norm = np.sqrt(ex**2 + ey**2)
        print(f"omega: {np.degrees(omega):3.0f} deg, e_norm: {e_norm:.5f}")


def fire_time_list(
        first_mid_time: datetime,
        window_start_bj: datetime,
        window_end_bj: datetime,
        period: float,
        fire_duration: float) -> list[datetime]:
    """计算同一纬度幅角的轨控时间列表"""
    # 找第一个轨控时间
    first_mid_time_bj = first_mid_time + timedelta(hours=8)
    for i in range(0, 40, 1):
        mid_time_temp = first_mid_time_bj + timedelta(seconds=period*i)
        if mid_time_temp >= window_start_bj:
            first_mid_time_bj = mid_time_temp
            break

    # 找后续轨控时间
    time_list = []
    for i in range(0, 40, 2):
        mid_time_bj = first_mid_time_bj + timedelta(seconds=period*i)

        if window_start_bj <= mid_time_bj <= window_end_bj:
            start_time_bj = mid_time_bj - timedelta(seconds=fire_duration/2)
            time_list.append(start_time_bj)

    return time_list


def generate_fire_time_file():
    """生成轨控策略文件"""
    window_start_bj = datetime.fromisoformat("2025-01-14 20:00:00.000")
    window_end_bj = datetime.fromisoformat("2025-01-15 12:00:00.000")
    orbit_period = 5710

    txt = ""
    for info in time_info:
        start_time_bj_list = fire_time_list(
            info["utc"],
            window_start_bj,
            window_end_bj,
            orbit_period,
            info["duration"])

        txt += f"# {info["sat_name"]} 星轨控策略\n"
        for index, time_bj in enumerate(start_time_bj_list):
            txt += (f"## 第 {index+1} 次轨控\n")
            txt += (f"  推进方向:     {info["direction"]}\n")
            txt += (f"  轨控开始时间: {time_bj.timestamp():.0f}s  (UTC+8: {time_bj})\n")
            txt += (f"  轨控结束时间: {(time_bj.timestamp() +
                    info["duration"]):.0f}s\n")
            txt += "\n"
        txt += "\n"

    with open(f"{window_start_bj.strftime("%Y%m%d")} 轨控.txt", "w", encoding='utf-8') as file:
        file.write(txt)


def generate_fire_time_file_csv(
        window_start_bj: datetime,
        window_end_bj: datetime,
        orbit_period: float):
    """生成轨控策略文件"""
    txt = ""
    for info in time_info:
        start_time_bj_list = fire_time_list(
            info["utc"],
            window_start_bj,
            window_end_bj,
            orbit_period,
            info["duration"])

        for _, time_bj in enumerate(start_time_bj_list):
            txt += f"{info["sat_name"]},"
            txt += (f"{info["direction"]},")
            txt += (f"{time_bj.timestamp():.0f},")
            txt += (f"{(time_bj.timestamp() + info["duration"]):.0f}")
            txt += "\n"

    with open(f"{window_start_bj.strftime("%Y%m%d")} 轨控.csv", "w", encoding='utf-8') as file:
        file.write(txt)

piesat2c = {
    "name": [
        {
            "sat_name": "A",
            "tag": "PIESAT02_C01",
        },
        {
            "sat_name": "B",
            "tag": "PIESAT02_C02",
        },
        {
            "sat_name": "C",
            "tag": "PIESAT02_C03",
        },
        {
            "sat_name": "D",
            "tag": "PIESAT02_C04",
        },
    ],

    "database": {
        "host_ip": "172.16.8.161",
        "host_port": 32728,
        "database": "measure",
        "measurement": "tm_all_PIESAT2C"
    },

    # 键名映射
    "key_mapping": {
        "TMK3001": "unix_time",
        "TMK3002": "a_mean",
        "TMK3003": "e_mean",
        "TMK3040": "reletive_valid",
        "TMK3041": "d_a",
        "TMK3042": "d_ex",
        "TMK3043": "d_ey",
        "TMK3044": "d_ix",
        "TMK3045": "d_iy",
        "TMK3046": "d_u",
    }
}

def rename_keys(data, key_mapping):
    """
    替换字典中键的命名
    :param data: 输入的字典或字典列表
    :param key_mapping: 键名映射字典，例如 {"old_key": "new_key"}
    :return: 替换键名后的字典或字典列表
    """
    if isinstance(data, dict):
        # 替换字典的键
        return {key_mapping.get(k, k): v for k, v in data.items()}
    elif isinstance(data, list):
        # 如果是列表，则逐个替换其中每个字典的键
        return [rename_keys(item, key_mapping) for item in data]
    else:
        # 如果数据不是字典或列表，直接返回
        return data


def get_data():
    """获取数据"""
    field = ", ".join(piesat2c["key_mapping"].keys())

    time_start = datetime.fromisoformat("2024-12-17 20:00:00.000")
    time_end = datetime.fromisoformat("2025-01-14 12:00:00.000")

    time_range = f" time > '{time_start.strftime(
        '%Y-%m-%dT%H:%M:%SZ')}' AND time < '{time_end.strftime('%Y-%m-%dT%H:%M:%SZ')}' "

    names = piesat2c["name"]
    for name in names:
        tag_key = f"_satelliteCode = '{name['tag']}'"
        data = influxdb1.get_database_points(
            piesat2c["database"],
            time_range,
            field,
            tag_key)

        updated_date = rename_keys(data, piesat2c["key_mapping"])

        csv_file.save_as_csv(f"{name['sat_name']}.csv", updated_date)


if __name__ == "__main__":
    # get_data()
    # anlyse_orbit()


    anlyse_orbit2()

    generate_fire_time_file_csv(
        window_start_bj=datetime.fromisoformat("2025-01-14 20:00:00.000"),
        window_end_bj=datetime.fromisoformat("2025-01-15 12:00:00.000"),
        orbit_period=5710)



import numpy as np

import os, sys
parent_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), '../../src'))
sys.path.append(parent_dir)
import aocs_lab.utils.lib as lib

# 上海厂房坐标 31.021018°N 121.393126°E
# 方舟实验室 40°03'16"N 116°21'08"E, 厂房偏转角度 asin(67/316) = 12.2°
# 太原坐标 38.8°N 111.6°E， 厂房偏转角度 asin(37.1/58.7)

def dms_to_decimal(deg: float, minutes: float = 0.0, seconds: float = 0.0, hemisphere: str | None = None) -> float:
    """
    将度分秒转换为度小数。
    - deg: 度（可为负）
    - minutes: 分（0~60）
    - seconds: 秒（0~60）
    - hemisphere: 方位字母，可为 'N','S','E','W'（大小写均可），如提供则决定正负

    返回：度小数（float）
    """
    if minutes < 0 or minutes >= 60 or seconds < 0 or seconds >= 60:
        raise ValueError("minutes 和 seconds 应满足 0 <= minutes < 60, 0 <= seconds < 60")

    # 由方位字母确定符号（若有）
    sign = 1
    if hemisphere:
        h = hemisphere.upper()
        if h not in ("N", "S", "E", "W"):
            raise ValueError("hemisphere 必须是 N/S/E/W")
        sign = -1 if h in ("S", "W") else 1

    # 如果未提供方位字母，则使用度的符号
    if hemisphere is None and deg < 0:
        sign = -1

    abs_deg = abs(float(deg))
    dec = abs_deg + float(minutes) / 60.0 + float(seconds) / 3600.0
    return sign * dec

if __name__ == "__main__":
    # 坐标定义，当地东北天L，厂房东北天F，卫星本体系B
    latitude_deg = dms_to_decimal(40, 3, 16, 'N')
    print(f'纬度 {latitude_deg}°')
    omega_L = lib.latitude_to_angular_velocity(np.deg2rad(latitude_deg))
    A_FL = lib.rotate_z(0)  # 当地东北天 到 厂房东北天坐标系
    A_BF = lib.rotate_z(np.deg2rad(12.2+90)) 
  
    omega_B = A_BF @ A_FL @ omega_L
    omega_B_deg = np.rad2deg(omega_B)

    print(f'卫星本体系理论角速度')
    print(f'x: {omega_B_deg[0]:10.6f} deg/s')
    print(f'y: {omega_B_deg[1]:10.6f} deg/s')
    print(f'z: {omega_B_deg[2]:10.6f} deg/s')



"""ats server api 调试"""
import json
import uuid
import time
import sys
import xml.etree.ElementTree as ET
import http.client
import requests
import signalr_sub


sys.stdout.reconfigure(encoding='utf-8')

# 启用 http.client 的调试
http.client.HTTPConnection.debuglevel = 0


def get_token(path):
    """
    获取 token
    """
    ams_server = "http://10.60.10.44:7000"
    url = ams_server + '/api/v1/user/login'
    data_user = {
        "name": "wangzhao",
        "password": "wangzhao"
    }

    response = requests.post(url, json=data_user, timeout=10)

    if response.status_code == 200:
        json_data = response.json()

        formatted_json = json.dumps(json_data, indent=4, ensure_ascii=False)
        print(formatted_json)

        with open(path, 'w', encoding='utf-8') as file:
            json.dump(json_data, file, indent=4)
    else:
        print(f"请求失败，状态码: {response.status_code}")


def read_token(path):
    """
    读取 token
    """
    with open(path, 'r', encoding='utf-8') as file:
        data = json.load(file)
    return data['data']['token']


def cmd(ats_server: str, token: str):
    """
    发送单条命令
    """
    url = ats_server + '/api/v1/Tc/Send'

    headers = {
        'accept': 'application/json',
        'Content-Type': 'application/json',
        'Authorization': f'Bearer {token}'
    }

    data = {
        "commandType": "TcPacket",
        "isSecret": False,
        "commandCode": "TCK2000",
        "commandParam": {
            "type": '0x99999999',
            "control_mode": '0x22222222',
        },
        "targetDevice": "CAN",
        "isJudge": False,
        "judgeTimeout": 10,
        "remark": "string",
        "canOption": {
            "canType": "Outer",
            "canBus": "A",
            "showConsole": True
        },
        "customBaseJudgeOption": {
            "isAddWithDefault": True,
            "judge": "string"
        },
        "customDiffJudgeOption": {
            "isAddWithDefault": True,
            "judge": "string"
        },
        "customParamJudgeOption": {
            "isAddWithDefault": True,
            "judge": "string"
        }
    }

    # response = requests.get(url, headers=headers, json=data, timeout=10)
    response = requests.post(url, headers=headers, json=data, timeout=10)

    # 检查请求是否成功
    if response.status_code == 200:
        json_data = response.json()

        formatted_json = json.dumps(json_data, indent=4, ensure_ascii=False)
        print(formatted_json)
    else:
        print(f"请求失败，状态码: {response.status_code}")


def cmd_list(ats_server: str, token: str, command_chain: dict, jobid: str):
    """
    发送指令链
    """
    url = ats_server + '/api/v1/Regulation/Start'

    headers = {
        'accept': 'application/json',
        'Content-Type': 'application/json',
        'Authorization': f'Bearer {token}'
    }

    data = {
        "jobId": jobid,
        "fileName": "...............",
        "test": "...............",
        "testId": "1",
        "isJudge": True,
        "ignoreJudgeFailure": False,
        "isSecret": False,
        "loop": 1,
        "id": "",
        "version": "",
        "targetDevice": "CAN",
        "commandType": "TcPacket",
        "canOption": {
            "canType": "Inner",
            "canBus": "A",
            "showConsole": True
        },
        "commandChain": [
            command_chain
        ],
        "recovery": ""
    }

    response = requests.post(url, headers=headers, json=data, timeout=10)

    # 检查请求是否成功
    if response.status_code == 200:
        # json_data = response.json()
        # print(json_data)
        pass
    else:
        print(f"指令链执行: 请求失败，状态码: {response.status_code}")


def cmd_list_get(ats_server: str, jobid: str):
    """获取任务执行信息ID列表"""
    url = ats_server + '/api/v1/Regulation/JobInfo' + f'/{jobid}'

    headers = {
        'accept': 'application/json',
        'Content-Type': 'application/json',
    }

    response = requests.get(url, headers=headers, timeout=10)

    # 检查请求是否成功
    cmd_list_over = True
    if response.status_code == 200:
        json_data = response.json()

        # formatted_json = json.dumps(json_data, indent=4, ensure_ascii=False)
        # print(formatted_json)

        if json_data['data'] is not None:
            if json_data['data']['status'] == 'Running':
                cmd_list_over = False
    else:
        print(f"指令链执行情况查询: 请求失败，状态码: {response.status_code}")
        cmd_list_over = True

    return cmd_list_over

def cmd_list_err_ignore(ats_server: str, jobid: str, token: str):
    """获取任务执行信息ID列表"""
    url = ats_server + '/api/v1/Job/Manage'

    headers = {
        'accept': 'application/json',
        'Content-Type': 'application/json',
        'Authorization': f'Bearer {token}'
    }

    data = {
        "jobId": jobid,
        "type": "Ignore"
    }

    response = requests.post(url, headers=headers, json=data, timeout=10)

    # 检查请求是否成功
    if response.status_code == 200:
        # json_data = response.json()
        # formatted_json = json.dumps(json_data, indent=4, ensure_ascii=False)
        # print(formatted_json)
        pass

    else:
        print(f"指令链判读失败忽略: 请求失败，状态码: {response.status_code}")



def parse_xml(xml_file: str):
    """
    解析 ats web 导出的指令链 xml 文件
    """
    tree = ET.parse(xml_file)
    root = tree.getroot()

    result = {
        'id': '',
        'version': '',
        "variables": [],
        'name': '',
        'commands': []
    }

    for _, command_chain in enumerate(root.find('CommandChains').findall('CommandChain')):
        chain = {
            'type': command_chain.get('Type'),
            'code': command_chain.get('Code'),
            'name': '...',
            'count': int(command_chain.get('Count')),
            'delayAfter': int(command_chain.get('Delay')),
            'judgeTimeout': int(command_chain.get('PostJudgeTimeout')),
            'params': {},
        }

        # 解析 Params
        params = command_chain.find('Params')
        for param in params.findall('Map'):
            chain['params'][param.get('Key')] = param.get('Value')

        # 解析 Judges
        judge = command_chain.find('BaseJudge')

        chain['customBaseJudge'] = {
            'judge': judge.get('Custom'),
            'isAddWithDefault': bool(judge.get('Logic') == 'AND')
        }

        judge = command_chain.find('DiffJudge')
        chain['customDiffJudge'] = {
            'judge': judge.get('Custom'),
            'isAddWithDefault': bool(judge.get('Logic') == 'AND')
        }

        judge = command_chain.find('ParamJudge')
        chain['customParamJudge'] = {
            'judge': judge.get('Custom'),
            'isAddWithDefault': bool(judge.get('Logic') == 'AND')
        }

        result['commands'].append(chain)

    # print(json.dumps(result, indent=4, ensure_ascii=False))

    return result

command_chain_all = [
    {
        "path": 'cmd_chain/自动化测试-1.xml',
        "uuid": "",
    },
    {
        "path": 'cmd_chain/自动化测试-2.xml',
        "uuid": "",
    }
]

ATS_SERVER = 'http://10.60.10.44:10701'

def on_message(messages):
    """处理接收到的消息"""
    for command_chain in command_chain_all:
        if messages[0] == command_chain['uuid']:
            # print(f"收到消息: {messages}")
            if messages[5] == 'Failed':
                cmd_list_err_ignore(ATS_SERVER, command_chain['uuid'], read_token('user.json'))

            if messages[5] == 'Successed' or messages[5] == 'Failed':
                print(f"cmd list: {command_chain['path']}, cmd index: {messages[2]}, status: {messages[5]}")



def main():
    """
    主函数
    """

    # cmd(ats_server, user_token)
    # cmd_list(ATS_SERVER, read_token('user.json'))

    # 读取 token, token 有效期为一周，超时需要重新获取
    # get_token('user.json')
    user_token = read_token('user.json')

    # 启动 signalr 订阅
    connection = signalr_sub.signalr_subscription(
        ATS_SERVER + "/pushhub",
        on_message)

    # 遍历发送指令链
    for index, chain in enumerate(command_chain_all):
        command_chain_json = parse_xml(chain['path'])

        jobid = str(uuid.uuid4())
        command_chain_all[index]['uuid'] = jobid

        cmd_list(ATS_SERVER,
                 user_token,
                 command_chain_json,
                 jobid)

        # 轮询队列执行状态，执行结束后开始下一指令链
        while True:
            time.sleep(1)
            if cmd_list_get(ATS_SERVER, jobid) is True:
                break

    # 停止 signalr 订阅
    connection.stop()





if __name__ == '__main__':
    main()



import numpy as np
import os, sys
from dataclasses import dataclass
from typing import TypeAlias

parent_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), '../../src'))
sys.path.append(parent_dir)
from aocs_lab.utils import lib

# 类型别名定义
UnitVector3D: TypeAlias = np.ndarray      # 单位三维向量
Dcm: TypeAlias = np.ndarray               # 3x3矩阵
Quaternion: TypeAlias = np.ndarray        # 四元数 [w, x, y, z]
AnglesDeg: TypeAlias = np.ndarray         # 角度列表（度）

# A 星布局报告输出
distribution_report_A = {
"dcm_SB_deg_starA": [
    [-0.707107,  0,         0.707107],
    [-0.298836,  0.906308, -0.298836],
    [-0.640856, -0.422618, -0.640856]],

"dcm_SB_deg_starB": [
    [ 0.707107,  0,         0.707107],
    [-0.298836, -0.906308,  0.298836],
    [ 0.640856, -0.422618, -0.640856]],

"theta_matrix_SB_deg_fiberA": [
        [  0,  90,  90],
        [ 90,  90, 180],
        [ 90,   0,  90]],

"theta_matrix_SB_deg_fiberB" : [
        [180, 90, 90],
        [ 90, 90,  0],
        [ 90,  0, 90]],

"theta_matrix_SB_deg_sun_sensor_A": [
    [  0,  90,  90],
    [ 90,  90,   0],
    [ 90, 180,  90]
],

"theta_matrix_SB_deg_sun_sensor_B": [
    [  0,  90,  90],
    [ 90,  90,   0],
    [ 90, 180,  90]
],

"theta_matrix_SB_deg_magmeter": [
        [180,  90,  90],
        [ 90, 180,  90],
        [ 90,  90,   0]],

"wheel_unit_vector_list": [
    [0.173648,-0.696364,-0.696364],
    [0.173648, 0.696364,-0.696364],
    [0.173648,-0.696364, 0.696364],
    [0.173648, 0.696364, 0.696364]
],

"magtorquer_unit_vector_list": [
    [-1,  0,  0],
    [ 0, -1,  0],
    [ 0,  0,  1]
],

"theta_matrix_SB_deg_trans_atenna_A": [
    [ 90, 150, 60],
    [  0,  90, 90],
    [ 90,  60, 30]
],

"theta_matrix_SB_deg_trans_atenna_B": [
    [ 90, 150, 60],
    [  0,  90, 90],
    [ 90,  60, 30]
]
}

# B 星布局报告输出
distribution_report_B = {
"theta_matrix_SB_deg_starA": [
    [ 60,        30,      90],
    [ 56.1742,  108.747,  40],
    [ 48.4392,  112.521, 130]],

"theta_matrix_SB_deg_starB": [
    [ 120,       30,       90],
    [  56.1742,  71.2528, 140],
    [ 131.561,  112.521,  130]],

"theta_matrix_SB_deg_fiber": [
        [180,  90,  90],
        [ 90, 180,  90],
        [ 90,  90,   0]],

"theta_matrix_SB_deg_mems" : [
        [ 90,  0, 90],
        [ 90, 90,  0],
        [ 0,  90, 90]],

"theta_matrix_SB_deg_sun_sensor_A_close": [
    [ 0, 90,  90],
    [90, 90, 180],
    [90,  0,  90]
],

"theta_matrix_SB_deg_sun_sensor_A_expand": [
    [180,  90,  90],
    [ 90,  90, 180],
    [ 90, 180,  90]
],

"theta_matrix_SB_deg_sun_sensor_B": [
    [180,  90,  90],
    [ 90,  90, 180],
    [ 90, 180,  90]
],

"theta_matrix_SB_deg_magmeter": [
        [ 90,  0,  90],
        [  0, 90,  90],
        [ 90, 90, 180]],

"wheel_vector_angle_deg_list": [
    [76.7559, 46.5044,133.4960],
    [76.7559, 46.5044, 46.5043],
    [76.7559,133.4960, 46.5043],
    [76.7559,133.4960,133.4960]
],

# todo 待确认
"magtorquer_vector_angle_deg_list": [
    [ 0, 90, 90],
    [90,  0, 90],
    [90, 90,  0]
],

"thruster_unit_vector_force": [
    [0,0,1]
],

"theta_matrix_SB_deg_trans_atenna_A": [
    [ 90, 150, 60],
    [  0,  90, 90],
    [ 90,  60, 30]
],

"theta_matrix_SB_deg_trans_atenna_B": [
    [  90, 30, 120],
    [ 180, 90,  90],
    [  90, 60,  30]
]
}

# CD 星布局报告输出
distribution_report_CD = {
# C01 D01
"theta_matrix_SB_deg_starA_C01_D01": [
    [  0,  90,  90],
    [ 90, 128, 142],
    [ 90,  38, 128]],

"theta_matrix_SB_deg_starB_C01_D01": [
    [  0,  90,  90],
    [ 90, 128,  38],
    [ 90, 142, 128]],

# C02 D02
"theta_matrix_SB_deg_starA_C02_D02": [
    [180, 90,  90],
    [ 90, 52,  38],
    [ 90, 38, 128]],

"theta_matrix_SB_deg_starB_C02_D02": [
    [180,  90,  90],
    [ 90,  52, 142],
    [ 90, 142, 128]],

# CD
"theta_matrix_SB_deg_fiber": [
        [180,  90,  90],
        [ 90, 180,  90],
        [ 90,  90,   0]],

"theta_matrix_SB_deg_mems" : [
        [ 90,  0, 90],
        [ 90, 90,  0],
        [ 0,  90, 90]],

"theta_matrix_SB_deg_sun_sensor_A_close": [
        [180,  90,  90],
        [ 90,  90, 180],
        [ 90, 180,  90]],

"theta_matrix_SB_deg_sun_sensor_A_expand": [
        [ 0, 90,  90],
        [90, 90, 180],
        [90,  0,  90]],

"theta_matrix_SB_deg_sun_sensor_B": [
        [ 0, 90,  90],
        [90, 90, 180],
        [90,  0,  90]],

"theta_matrix_SB_deg_magmeter": [
        [ 90,  90, 180],
        [ 90, 180,  90],
        [180,  90,  90]],

"wheel_vector_angle_deg_list": [
    [76.7559, 46.5044, 46.5043],
    [76.7559,133.4960, 46.5043],
    [76.7559, 46.5044,133.4960],
    [76.7559,133.4960,133.4960]
],

"magtorquer_vector_angle_deg_list": [
    [ 0, 90, 90],
    [90,  0, 90],
    [90, 90,  0]
],

"thruster_unit_vector_force": [
    [0,0,1]
],

"theta_matrix_SB_deg_trans_atenna_A": [
    [ 90,  30,  60],
    [180,  90,  90],
    [ 90, 120,  30]
],

"theta_matrix_SB_deg_trans_atenna_B": [
    [ 90, 150, 120],
    [  0,  90,  90],
    [ 90, 120,  30]
]

}

@dataclass
class device_install:
    star_A: Quaternion
    star_B: Quaternion
    gyro_A: Dcm
    gyro_B: Dcm
    sun_sensor_A_close: Dcm
    sun_sensor_A_expand: Dcm
    sun_sensor_B: Dcm
    magmeter_A: Dcm
    magmeter_B: Dcm
    wheel_A: UnitVector3D
    wheel_B: UnitVector3D
    wheel_C: UnitVector3D
    wheel_D: UnitVector3D
    magtorquer_A: UnitVector3D
    magtorquer_B: UnitVector3D
    magtorquer_C: UnitVector3D
    thruster_A: UnitVector3D
    trans_antenna_A: Dcm
    trans_antenna_B: Dcm

    def save_results_to_file(self, filename: str = "satellite_install_results.txt"):
        """将计算结果保存到文本文件"""
        with open(filename, 'w', encoding='utf-8') as f:
            f.write("# 卫星传感器安装矩阵计算结果\n\n")
            
            # 获取所有成员变量
            import dataclasses
            import numpy as np
            for field in dataclasses.fields(self):
                field_name = field.name
                value = getattr(self, field_name)
                
                f.write(f"{field_name}:\n")
                
                # 根据numpy数组的shape判断类型并格式化输出
                if hasattr(value, 'shape'):
                    if value.shape == (3, 3):
                        # 3x3矩阵 (方向余弦矩阵)
                        f.write("方向余弦矩阵 (单机系到本体系):\n")
                        for row in value:
                            row_str = ", ".join([f"{x:.8f}" for x in row])
                            f.write(f"  [{row_str}]\n")
                            
                    elif value.shape == (4,):
                        # 4元素1维数组 (四元数)
                        quat_str = ", ".join([f"{x:.15f}" for x in value])
                        f.write(f"四元数 [w, x, y, z] (单机系到本体系):\n [{quat_str}]\n")
                        
                    elif value.shape == (3,):
                        # 3元素1维数组 (三维向量)
                        vector_str = ", ".join([f"{x:.8f}" for x in value])
                        f.write(f"单位方向矢量 (本体系): [{vector_str}]\n")
                    else:
                        # 其他shape的numpy数组
                        f.write(f"numpy数组 (shape: {value.shape}): {value}\n")
                        
                else:
                    # 其他类型
                    raise ValueError(f"不支持的类型: {type(value)}")

                f.write("\n")
        
        print(f"计算结果已保存到文件: {filename}")

def star_tracker_quat(theta_matrix_SB_deg: list[list[float]]) -> Quaternion:
    dcm_SB = lib.theta_deg_to_cos_matrix(theta_matrix_SB_deg)
    err = lib.orthogonality_error(dcm_SB)
    dcm_BS = dcm_SB.T
    q_BS = lib.dcm2quat(dcm_BS)

    print("Orthogonality error", err)

    return q_BS

def star_tracker_quat_from_dcm(dcm_SB: np.ndarray) -> Quaternion:
    err = lib.orthogonality_error(dcm_SB)
    dcm_BS = dcm_SB.T
    q_BS = lib.dcm2quat(dcm_BS)

    print("Orthogonality error", err)

    return q_BS


def sensor_3axis_dcm(theta_matrix_SB_deg: list) -> Dcm:
    dcm_SB = lib.theta_deg_to_cos_matrix(theta_matrix_SB_deg)
    err = lib.orthogonality_error(dcm_SB)
    dcm_BS = dcm_SB.T

    print("Orthogonality error", err)

    return dcm_BS

def actuator_unit_vector(angle_deg: list) -> UnitVector3D:
    vector = np.cos(np.deg2rad(angle_deg))

    return vector

lz07_a01 = device_install(
    star_A=star_tracker_quat_from_dcm(np.array(distribution_report_A['dcm_SB_deg_starA'])),
    star_B=star_tracker_quat_from_dcm(np.array(distribution_report_A['dcm_SB_deg_starB'])),
    gyro_A=sensor_3axis_dcm(distribution_report_A['theta_matrix_SB_deg_fiberA']),
    gyro_B=sensor_3axis_dcm(distribution_report_A['theta_matrix_SB_deg_fiberB']),
    sun_sensor_A_close=sensor_3axis_dcm(distribution_report_A['theta_matrix_SB_deg_sun_sensor_A']),
    sun_sensor_A_expand=sensor_3axis_dcm(distribution_report_A['theta_matrix_SB_deg_sun_sensor_A']),
    sun_sensor_B=sensor_3axis_dcm(distribution_report_A['theta_matrix_SB_deg_sun_sensor_B']),
    magmeter_A=sensor_3axis_dcm(distribution_report_A['theta_matrix_SB_deg_magmeter']),
    magmeter_B=sensor_3axis_dcm(distribution_report_A['theta_matrix_SB_deg_magmeter']),
    wheel_A=np.array(distribution_report_A['wheel_unit_vector_list'][0]),
    wheel_B=np.array(distribution_report_A['wheel_unit_vector_list'][1]),
    wheel_C=np.array(distribution_report_A['wheel_unit_vector_list'][2]),
    wheel_D=np.array(distribution_report_A['wheel_unit_vector_list'][3]),
    magtorquer_A=np.array(distribution_report_A['magtorquer_unit_vector_list'][0]),
    magtorquer_B=np.array(distribution_report_A['magtorquer_unit_vector_list'][1]),
    magtorquer_C=np.array(distribution_report_A['magtorquer_unit_vector_list'][2]),
    thruster_A=np.array([0,0,0]),
    trans_antenna_A=sensor_3axis_dcm(distribution_report_A['theta_matrix_SB_deg_trans_atenna_A']),
    trans_antenna_B=sensor_3axis_dcm(distribution_report_A['theta_matrix_SB_deg_trans_atenna_B']),
)


lz07_b01 = device_install(
    star_A=star_tracker_quat(distribution_report_B['theta_matrix_SB_deg_starA']),
    star_B=star_tracker_quat(distribution_report_B['theta_matrix_SB_deg_starB']),
    gyro_A=sensor_3axis_dcm(distribution_report_B['theta_matrix_SB_deg_fiber']),
    gyro_B=sensor_3axis_dcm(distribution_report_B['theta_matrix_SB_deg_mems']),
    sun_sensor_A_close=sensor_3axis_dcm(distribution_report_B['theta_matrix_SB_deg_sun_sensor_A_close']),
    sun_sensor_A_expand=sensor_3axis_dcm(distribution_report_B['theta_matrix_SB_deg_sun_sensor_A_expand']),
    sun_sensor_B=sensor_3axis_dcm(distribution_report_B['theta_matrix_SB_deg_sun_sensor_B']),
    magmeter_A=sensor_3axis_dcm(distribution_report_B['theta_matrix_SB_deg_magmeter']),
    magmeter_B=sensor_3axis_dcm(distribution_report_B['theta_matrix_SB_deg_magmeter']),
    wheel_A=actuator_unit_vector(distribution_report_B['wheel_vector_angle_deg_list'][0]),
    wheel_B=actuator_unit_vector(distribution_report_B['wheel_vector_angle_deg_list'][1]),
    wheel_C=actuator_unit_vector(distribution_report_B['wheel_vector_angle_deg_list'][2]),
    wheel_D=actuator_unit_vector(distribution_report_B['wheel_vector_angle_deg_list'][3]),
    magtorquer_A=actuator_unit_vector(distribution_report_B['magtorquer_vector_angle_deg_list'][0]),
    magtorquer_B=actuator_unit_vector(distribution_report_B['magtorquer_vector_angle_deg_list'][1]),
    magtorquer_C=actuator_unit_vector(distribution_report_B['magtorquer_vector_angle_deg_list'][2]),
    thruster_A=np.array(distribution_report_B['thruster_unit_vector_force'][0]),
    trans_antenna_A=sensor_3axis_dcm(distribution_report_B['theta_matrix_SB_deg_trans_atenna_A']),
    trans_antenna_B=sensor_3axis_dcm(distribution_report_B['theta_matrix_SB_deg_trans_atenna_B']),
)

lz07_c01 = device_install(
    star_A=star_tracker_quat(distribution_report_CD['theta_matrix_SB_deg_starA_C01_D01']),
    star_B=star_tracker_quat(distribution_report_CD['theta_matrix_SB_deg_starB_C01_D01']),
    gyro_A=sensor_3axis_dcm(distribution_report_CD['theta_matrix_SB_deg_fiber']),
    gyro_B=sensor_3axis_dcm(distribution_report_CD['theta_matrix_SB_deg_mems']),
    sun_sensor_A_close=sensor_3axis_dcm(distribution_report_CD['theta_matrix_SB_deg_sun_sensor_A_close']),
    sun_sensor_A_expand=sensor_3axis_dcm(distribution_report_CD['theta_matrix_SB_deg_sun_sensor_A_expand']),
    sun_sensor_B=sensor_3axis_dcm(distribution_report_CD['theta_matrix_SB_deg_sun_sensor_B']),
    magmeter_A=sensor_3axis_dcm(distribution_report_CD['theta_matrix_SB_deg_magmeter']),
    magmeter_B=sensor_3axis_dcm(distribution_report_CD['theta_matrix_SB_deg_magmeter']),
    wheel_A=actuator_unit_vector(distribution_report_CD['wheel_vector_angle_deg_list'][0]),
    wheel_B=actuator_unit_vector(distribution_report_CD['wheel_vector_angle_deg_list'][1]),
    wheel_C=actuator_unit_vector(distribution_report_CD['wheel_vector_angle_deg_list'][2]),
    wheel_D=actuator_unit_vector(distribution_report_CD['wheel_vector_angle_deg_list'][3]),
    magtorquer_A=actuator_unit_vector(distribution_report_CD['magtorquer_vector_angle_deg_list'][0]),
    magtorquer_B=actuator_unit_vector(distribution_report_CD['magtorquer_vector_angle_deg_list'][1]),
    magtorquer_C=actuator_unit_vector(distribution_report_CD['magtorquer_vector_angle_deg_list'][2]),
    thruster_A=np.array(distribution_report_CD['thruster_unit_vector_force'][0]),
    trans_antenna_A=sensor_3axis_dcm(distribution_report_CD['theta_matrix_SB_deg_trans_atenna_A']),
    trans_antenna_B=sensor_3axis_dcm(distribution_report_CD['theta_matrix_SB_deg_trans_atenna_B']),
)

lz07_c02 = device_install(
    star_A=star_tracker_quat(distribution_report_CD['theta_matrix_SB_deg_starA_C02_D02']),
    star_B=star_tracker_quat(distribution_report_CD['theta_matrix_SB_deg_starB_C02_D02']),
    gyro_A=sensor_3axis_dcm(distribution_report_CD['theta_matrix_SB_deg_fiber']),
    gyro_B=sensor_3axis_dcm(distribution_report_CD['theta_matrix_SB_deg_mems']),
    sun_sensor_A_close=sensor_3axis_dcm(distribution_report_CD['theta_matrix_SB_deg_sun_sensor_A_close']),
    sun_sensor_A_expand=sensor_3axis_dcm(distribution_report_CD['theta_matrix_SB_deg_sun_sensor_A_expand']),
    sun_sensor_B=sensor_3axis_dcm(distribution_report_CD['theta_matrix_SB_deg_sun_sensor_B']),
    magmeter_A=sensor_3axis_dcm(distribution_report_CD['theta_matrix_SB_deg_magmeter']),
    magmeter_B=sensor_3axis_dcm(distribution_report_CD['theta_matrix_SB_deg_magmeter']),
    wheel_A=actuator_unit_vector(distribution_report_CD['wheel_vector_angle_deg_list'][0]),
    wheel_B=actuator_unit_vector(distribution_report_CD['wheel_vector_angle_deg_list'][1]),
    wheel_C=actuator_unit_vector(distribution_report_CD['wheel_vector_angle_deg_list'][2]),
    wheel_D=actuator_unit_vector(distribution_report_CD['wheel_vector_angle_deg_list'][3]),
    magtorquer_A=actuator_unit_vector(distribution_report_CD['magtorquer_vector_angle_deg_list'][0]),
    magtorquer_B=actuator_unit_vector(distribution_report_CD['magtorquer_vector_angle_deg_list'][1]),
    magtorquer_C=actuator_unit_vector(distribution_report_CD['magtorquer_vector_angle_deg_list'][2]),
    thruster_A=np.array(distribution_report_CD['thruster_unit_vector_force'][0]),
    trans_antenna_A=sensor_3axis_dcm(distribution_report_CD['theta_matrix_SB_deg_trans_atenna_A']),
    trans_antenna_B=sensor_3axis_dcm(distribution_report_CD['theta_matrix_SB_deg_trans_atenna_B']),
)

lz07_d01 = device_install(
    star_A=star_tracker_quat(distribution_report_CD['theta_matrix_SB_deg_starA_C01_D01']),
    star_B=star_tracker_quat(distribution_report_CD['theta_matrix_SB_deg_starB_C01_D01']),
    gyro_A=sensor_3axis_dcm(distribution_report_CD['theta_matrix_SB_deg_fiber']),
    gyro_B=sensor_3axis_dcm(distribution_report_CD['theta_matrix_SB_deg_mems']),
    sun_sensor_A_close=sensor_3axis_dcm(distribution_report_CD['theta_matrix_SB_deg_sun_sensor_A_close']),
    sun_sensor_A_expand=sensor_3axis_dcm(distribution_report_CD['theta_matrix_SB_deg_sun_sensor_A_expand']),
    sun_sensor_B=sensor_3axis_dcm(distribution_report_CD['theta_matrix_SB_deg_sun_sensor_B']),
    magmeter_A=sensor_3axis_dcm(distribution_report_CD['theta_matrix_SB_deg_magmeter']),
    magmeter_B=sensor_3axis_dcm(distribution_report_CD['theta_matrix_SB_deg_magmeter']),
    wheel_A=actuator_unit_vector(distribution_report_CD['wheel_vector_angle_deg_list'][0]),
    wheel_B=actuator_unit_vector(distribution_report_CD['wheel_vector_angle_deg_list'][1]),
    wheel_C=actuator_unit_vector(distribution_report_CD['wheel_vector_angle_deg_list'][2]),
    wheel_D=actuator_unit_vector(distribution_report_CD['wheel_vector_angle_deg_list'][3]),
    magtorquer_A=actuator_unit_vector(distribution_report_CD['magtorquer_vector_angle_deg_list'][0]),
    magtorquer_B=actuator_unit_vector(distribution_report_CD['magtorquer_vector_angle_deg_list'][1]),
    magtorquer_C=actuator_unit_vector(distribution_report_CD['magtorquer_vector_angle_deg_list'][2]),
    thruster_A=np.array(distribution_report_CD['thruster_unit_vector_force'][0]),
    trans_antenna_A=sensor_3axis_dcm(distribution_report_CD['theta_matrix_SB_deg_trans_atenna_A']),
    trans_antenna_B=sensor_3axis_dcm(distribution_report_CD['theta_matrix_SB_deg_trans_atenna_B']),
)

lz07_d02 = device_install(
    star_A=star_tracker_quat(distribution_report_CD['theta_matrix_SB_deg_starA_C02_D02']),
    star_B=star_tracker_quat(distribution_report_CD['theta_matrix_SB_deg_starB_C02_D02']),
    gyro_A=sensor_3axis_dcm(distribution_report_CD['theta_matrix_SB_deg_fiber']),
    gyro_B=sensor_3axis_dcm(distribution_report_CD['theta_matrix_SB_deg_mems']),
    sun_sensor_A_close=sensor_3axis_dcm(distribution_report_CD['theta_matrix_SB_deg_sun_sensor_A_close']),
    sun_sensor_A_expand=sensor_3axis_dcm(distribution_report_CD['theta_matrix_SB_deg_sun_sensor_A_expand']),
    sun_sensor_B=sensor_3axis_dcm(distribution_report_CD['theta_matrix_SB_deg_sun_sensor_B']),
    magmeter_A=sensor_3axis_dcm(distribution_report_CD['theta_matrix_SB_deg_magmeter']),
    magmeter_B=sensor_3axis_dcm(distribution_report_CD['theta_matrix_SB_deg_magmeter']),
    wheel_A=actuator_unit_vector(distribution_report_CD['wheel_vector_angle_deg_list'][0]),
    wheel_B=actuator_unit_vector(distribution_report_CD['wheel_vector_angle_deg_list'][1]),
    wheel_C=actuator_unit_vector(distribution_report_CD['wheel_vector_angle_deg_list'][2]),
    wheel_D=actuator_unit_vector(distribution_report_CD['wheel_vector_angle_deg_list'][3]),
    magtorquer_A=actuator_unit_vector(distribution_report_CD['magtorquer_vector_angle_deg_list'][0]),
    magtorquer_B=actuator_unit_vector(distribution_report_CD['magtorquer_vector_angle_deg_list'][1]),
    magtorquer_C=actuator_unit_vector(distribution_report_CD['magtorquer_vector_angle_deg_list'][2]),
    thruster_A=np.array(distribution_report_CD['thruster_unit_vector_force'][0]),
    trans_antenna_A=sensor_3axis_dcm(distribution_report_CD['theta_matrix_SB_deg_trans_atenna_A']),
    trans_antenna_B=sensor_3axis_dcm(distribution_report_CD['theta_matrix_SB_deg_trans_atenna_B']),
)


def main():
    lz07_a01.save_results_to_file("satellite_install_results_A01.txt")
    lz07_b01.save_results_to_file("satellite_install_results_B01.txt")
    lz07_c01.save_results_to_file("satellite_install_results_C01.txt")
    lz07_c02.save_results_to_file("satellite_install_results_C02.txt")
    lz07_d01.save_results_to_file("satellite_install_results_D01.txt")
    lz07_d02.save_results_to_file("satellite_install_results_D02.txt")

if __name__ == "__main__":
    main()


"""灵知 05 发射时间线绘制"""
from datetime import datetime, timedelta
import matplotlib.pyplot as plt
from matplotlib import rcParams
import matplotlib.dates as mdates
from matplotlib.axes import Axes

# 阳照区开始
sun_start = [
    datetime(2024, 12, 17, 3, 16, 50),
    datetime(2024, 12, 17, 4, 52, 11),
    datetime(2024, 12, 17, 6, 27, 32),
    datetime(2024, 12, 17, 8, 2, 54)
]

# 阳照区结束
sun_end = [
    datetime(2024, 12, 17, 4, 18, 46),
    datetime(2024, 12, 17, 5, 54, 7),
    datetime(2024, 12, 17, 7, 29, 28),
    datetime(2024, 12, 17, 9, 4, 49)
]

# 升交点时间
anode_time = [
    datetime(2024, 12, 17, 3, 53, 50),
    datetime(2024, 12, 17, 5, 29, 20),
    datetime(2024, 12, 17, 7, 4, 40),
    datetime(2024, 12, 17, 8, 40, 0)
]

# 卫星事件点
sat_dates0 = [
    {
        "text": "发射",
        "time": datetime(2024, 12, 17, 2, 50, 47)
    }
]

sat_dates1 = [
    {
        "text": "C01 星箭分离 (T0)",
        "time": datetime(2024, 12, 17, 3, 4, 51)
    },
    {
        "text": "\n\n\n\nC02 星箭分离",
        "time": datetime(2024, 12, 17, 3, 4, 52)
    },
    {
        "text": "C03 星箭分离",
        "time": datetime(2024, 12, 17, 3, 5, 26)
    },
    {
        "text": "C04 星箭分离",
        "time": datetime(2024, 12, 17, 3, 5, 56)
    }
]

sat_dates2 = [
    {
        "text": "C01 最早展帆板时间 (T0+220s)",
        "time": sat_dates1[0]['time'] + timedelta(seconds=220)
    },
    {
        "text": "C01 最晚展帆板时间 (T0+250s)\n(初始角速度 2 deg/s)",
        "time": sat_dates1[0]['time'] + timedelta(seconds=250)
    },
    {
        "text": "C01 最早进对日一阶段时间 (T0+370s)",
        "time": sat_dates1[0]['time'] + timedelta(seconds=220) + timedelta(seconds=150)
    },
    {
        "text": "C03 最早展帆板时间",
        "time": sat_dates1[2]['time'] + timedelta(seconds=220)
    },
    {
        "text": "C04 最早展帆板时间",
        "time": sat_dates1[3]['time'] + timedelta(seconds=220)
    },
    # {
    #     "text": "C04 最晚展帆板时间\n(初始角速度 2 deg/s)",
    #     "time": sat_dates1[3]['time'] + timedelta(seconds=250)
    # },
    # {
    #     "text": "C03 最早进对日一阶段时间",
    #     "time": sat_dates1[3]['time'] + timedelta(seconds=220) + timedelta(seconds=150)
    # },
]


#  过站时间线
station_time = [
    {
        "name": "马来站",
        "task": "确认星箭分离\n进阻尼三阶段\n帆板解锁",
        "time":
        [
            datetime.fromisoformat("2024-12-17 03:04:51"),
            datetime.fromisoformat("2024-12-17 03:09:42"),
        ]
    },
    # {
    #     "name": "阿根廷站",
    #     "task": "确认对日四阶段（自旋对日）\nSAR 天线展开",
    #     "time":
    #     [
    #         datetime(2024, 12, 17, 3, 34, 32),
    #         datetime(2024, 12, 17, 3, 44, 48)
    #     ]
    # },
    # {
    #     "name": "中西部地面站",
    #     "task": "确认对日四阶段（自旋对日）\nSAR 天线展开",
    #     "time":
    #     [
    #         datetime(2024, 12, 17, 4, 23, 34),
    #         datetime(2024, 12, 17, 4, 37, 59)
    #     ]
    # },
    # {
    #     "name": "阿根廷站",
    #     "task": "星敏对准、校时、组合开启",
    #     "time":
    #     [
    #         datetime(2024, 12, 17, 5, 10, 4),
    #         datetime(2024, 12, 17, 5, 15, 58)
    #     ]
    # },
    # {
    #     "name": "中西部地面站\n一星一站跟踪",
    #     "task": "星敏接入\n转约束对日",
    #     "time":
    #     [
    #         datetime(2024, 12, 17, 5, 59, 16),
    #         datetime(2024, 12, 17, 6, 10, 7)
    #     ]
    # },
    # {
    #     "name": "阿塞拜疆",
    #     "task": "星敏对准、校时、组合开启\n陀螺温补，零位设置",
    #     "time":
    #     [
    #         datetime(2024, 12, 17, 7, 35, 27),
    #         datetime(2024, 12, 17, 7, 45, 2)
    #     ]
    # },
    # {
    #     "name": "南非",
    #     "time":
    #     [
    #         datetime(2024, 12, 17, 7, 56, 45),
    #         datetime(2024, 12, 17, 8, 5, 42)
    #     ]
    # }
]


def plot_time_line_of_station(plt_ax: Axes, dates: datetime,
                              station_name: str, task: str,
                              color="red", linewidth=5):
    """绘制过站时间线"""
    # 绘制时间线
    plt_ax.hlines(y=-0.1, xmin=min(dates), xmax=max(dates),
              color=color, linewidth=linewidth, zorder=3)
    # 添加站点标签
    plt_ax.text(dates[0], -0.4, dates[0].strftime("%H:%M:%S"),
            ha="right", va="bottom", fontsize=10, rotation=45)
    plt_ax.text(dates[1], -0.4, dates[1].strftime("%H:%M:%S"),
            ha="right", va="bottom", fontsize=10, rotation=45)
    dt = dates[1] - dates[0]
    minutes, seconds = divmod(dt.seconds, 60)
    plt_ax.text(dates[0], -0.6,  f'{station_name}\n{minutes}m{seconds}s',
            ha="left", va="bottom", fontsize=10)
    plt_ax.text(dates[0], -0.8,  f'{task}', ha="left",
            va="bottom", fontsize=10, color="red")


def plot_time_line_of_sat(plt_ax: Axes, times: list[dict]):
    """绘制卫星时间线"""
    timeline = [time["time"] for time in times]
    plt_ax.hlines(y=0, xmin=min(timeline), xmax=max(timeline),
              color="gray", linewidth=1.5)
    plt_ax.scatter(timeline, [0] * len(timeline), color="blue", zorder=2)

    for time in times:
        text = f"{time['text']}\n{time['time'].strftime('%H:%M:%S')}"
        plt_ax.text(time['time'], 0.2, text,
                ha="left", va="bottom", fontsize=10, rotation=45)


if __name__ == "__main__":
    # 全局设置字体为 SimHei
    rcParams['font.sans-serif'] = ['SimHei']  # 使用黑体
    rcParams['axes.unicode_minus'] = False   # 解决负号显示问题

    # 创建图形
    fig, ax = plt.subplots(figsize=(20, 6))

    # 绘制卫星时间线
    plot_time_line_of_sat(ax, sat_dates1 + sat_dates2)

    # 绘制过站时间线
    for i in range(len(station_time)-6):
        plot_time_line_of_station(
            ax, station_time[i]["time"], station_time[i]["name"], station_time[i]["task"])

    # 阳照区绘制
    for i in range(0):
        ax.axvspan(sun_start[i], sun_end[i],
                   color="yellow", alpha=0.5, zorder=1, ymin=0, ymax=0.5)  # 矩形块
        ax.text(sun_start[i], -0.95,
                f'阳照区时间\n{
                    sun_start[i].strftime("%H:%M:%S")} ~ {sun_end[i].strftime("%H:%M:%S")}',
            ha="left", va="bottom", fontsize=10)

    # 圈次绘制
    for i in range(0):
        ax.axvspan(anode_time[i], anode_time[i+1],
                   color="lightblue", alpha=0.5, zorder=1, ymin=0.5, ymax=1)  # 矩形块
        ax.text(anode_time[i], 0.8,
                f'升交点时间\n{anode_time[i].strftime("%H:%M:%S")}\n圈次计数: {i+1}',
                ha="left", va="bottom", fontsize=10)

    # 设置 x 轴日期格式
    date_format = mdates.DateFormatter('%Y-%m-%d\n%H:%M:%S\n%A')  # 自定义日期格式
    ax.xaxis.set_major_formatter(date_format)
    fig.autofmt_xdate()  # 自动调整日期标签角度

    # 美化时间线
    ax.set_ylim(-1, 1)
    ax.set_yticks([])  # 隐藏 y 轴刻度
    ax.grid(visible=False)  # 不显示网格线
    plt.tight_layout()

    # 显示图形
    plt.savefig("timeline_highres.png", dpi=300,
                bbox_inches="tight")  # 设置分辨率和边框
    # plt.show()



"""分析对日姿态的能源效率"""
import os
import sys
import numpy as np
import matplotlib.pyplot as plt
from scipy.spatial.transform import Rotation

parent_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), '../src'))
sys.path.append(parent_dir)
from aocs_lab.utils.lib import vector_angle, orbit_period, euler2dcm
import aocs_lab.utils.constants as constants

SEMIMAJAR_AXIS = 6913.49e3  # 卫星轨道半长轴
SAMPLE_TIME = 10  # 采样时间
sample_num = int(orbit_period(SEMIMAJAR_AXIS) / SAMPLE_TIME)  # 一轨周期采样点数

def sun_cone_in_vvlh(beta_angle: float) -> list:
    """
    计算太阳矢量在VVLH系的圆锥遍历
    返回太阳单位矢量列表
    """

    phi_line = np.linspace(0, 2*np.pi, sample_num)

    v_sun_cone = []
    for phi in phi_line:
        y = np.sin(beta_angle)
        xz = np.cos(beta_angle)
        v_sun = np.array([xz*np.sin(phi), y, xz*np.cos(phi)])

        v_sun_cone.append(v_sun)

    return v_sun_cone


def sun_vector_in_light(v_sun_cone: list, semimajor_axis: float) -> list:
    """计算太阳矢量在光照区遍历"""
    v_earth = np.array([0, 0, 1])
    v_sun_in_light_index = []
    for index, v_sun in enumerate(v_sun_cone):
        earth_sun_angle = vector_angle(v_earth, v_sun)
        if earth_sun_angle > np.arcsin(constants.EARTH_EQUATORIAL_RADIUS/semimajor_axis):
            v_sun_in_light_index.append(index)

    return v_sun_in_light_index

def calc_panel_normal(v_sun_unit: np.ndarray) -> np.ndarray:
    """计算太阳帆板法向"""
    v_earth_unit = np.array([0,0,1])

    sun_on_z = np.dot(v_earth_unit, v_sun_unit) * v_earth_unit
    sun_on_xy_plane = v_sun_unit - sun_on_z
    sun_on_xy_plane_unit = sun_on_xy_plane / np.linalg.norm(sun_on_xy_plane)

    if v_sun_unit[2] < 0:
        v_panel = 2.2*v_sun_unit + sun_on_xy_plane_unit
    else:
        v_panel = v_sun_unit + 3*sun_on_xy_plane_unit
    # v_panel[0] = np.abs(v_panel[0]) * v_panel[0]
    v_panel_unit = v_panel / np.linalg.norm(v_panel)

    return v_panel_unit

def calc_sun_att(v_panel_normal_unit: np.ndarray) -> Rotation:
    """计算约束对日姿态"""
    v_earth_unit = np.array([0, 0, 1])

    y_unit = - v_panel_normal_unit
    x = np.cross(y_unit, v_earth_unit)
    x_unit = x / np.linalg.norm(x)
    z = np.cross(x_unit, y_unit)
    z_unit = z / np.linalg.norm(z)

    dcm_bo = np.array([x_unit, y_unit, z_unit])
    # rot = Rotation.from_matrix([x_unit, y_unit, z_unit])

    # print(rot.as_rotvec(degrees=True))

    return dcm_bo

def calc_att_angular_rate(att_bo: np.ndarray, att_blast_o: np.ndarray, dt: float) -> np.ndarray:
    """计算姿态角速度"""
    att_b_blast =  att_bo @ att_blast_o.transpose()
    rot = Rotation.from_matrix(att_b_blast)
    att_rate = rot.as_rotvec() / dt

    return att_rate

def calc_att_angular_acceleration(
        att_rate: np.ndarray,
        att_rate_last: np.ndarray,
        dt: float) -> np.ndarray:
    """计算姿态角加速度"""
    att_acceleration = (att_rate - att_rate_last) / dt

    return att_acceleration


def panel_normal_cone(v_sun_cone: list) -> list:
    """计算太阳帆板法向圆锥"""
    panel_n_cone = []
    for v_sun in v_sun_cone:
        panel_normal = calc_panel_normal(v_sun)
        panel_n_cone.append(panel_normal)

    return panel_n_cone


def energy_efficiency(
        sun_cone: list,
        sun_cone_in_light_index: list,
        panel_n_cone: list) -> float:
    """计算能源效率"""
    sun_cone_in_light = [sun_cone[i] for i in sun_cone_in_light_index]
    panel_normal_cone_in_light = [panel_n_cone[i] for i in sun_cone_in_light_index]

    energy_one_orbit = 0
    for v_sun in sun_cone:
        energy_one_orbit += 1

    energy_in_light = 0
    for v_sun in sun_cone_in_light:
        energy_in_light += 1

    energy_real = 0
    for panel_normal, v_sun in zip(panel_normal_cone_in_light, sun_cone_in_light):
        energy_real += np.cos(vector_angle(v_sun, panel_normal))

    print(f"光照区能量总量 与 一轨能量之比: {100 * energy_in_light/energy_one_orbit:.1f} %")
    print(f"实际能量 与 光照区能量总量之比: {100 * energy_real/energy_in_light:.1f} %")

def anlyse_sun_att(panel_n_cone: list):
    """分析对日姿态"""
    att_last = Rotation.from_euler('ZYX', [0, 0, 0]).as_matrix()
    att_rate_last = np.array([0, 0, 0])

    att_rate_list = []
    att_acceleration_list = []

    for index, v_panel in enumerate(panel_n_cone) :
        att_bo = calc_sun_att(v_panel)
        att_rate = calc_att_angular_rate(att_bo, att_last, SAMPLE_TIME)
        att_acceleration = calc_att_angular_acceleration(att_rate, att_rate_last, SAMPLE_TIME)

        if index > 0:
            att_rate_list.append(att_rate)

        if index > 1:
            att_acceleration_list.append(att_acceleration)

        att_last = att_bo
        att_rate_last = att_rate

    timeline = np.linspace(0, SAMPLE_TIME*len(att_rate_list), len(att_rate_list))
    print(len(att_rate_list))
    _, ax = plt.subplots(2, 1)
    ax[0].plot(timeline, np.rad2deg(att_rate_list))
    ax[0].set_title('Attitude Rate')
    ax[0].set_xlabel('Time (s)')
    ax[0].set_ylabel('Attitude Rate (deg/s)')
    ax[0].grid()
    ax[0].legend(['x', 'y', 'z'])
    # mark_max_min(ax[0], timeline, np.rad2deg(att_rate_list))
    plt.subplots_adjust(hspace=0.35)  # 增加两幅图的纵向间距
    timeline = np.linspace(0, SAMPLE_TIME*len(att_acceleration_list), len(att_acceleration_list))
    ax[1].plot(timeline, np.rad2deg(att_acceleration_list))
    ax[1].set_title('Attitude Acceleration')
    ax[1].set_xlabel('Time (s)')
    ax[1].set_ylabel('Attitude Acceleration (deg/s^2)')
    ax[1].legend(['x', 'y', 'z'])
    ax[1].grid()
    # mark_max_min(ax[1], timeline, np.rad2deg(att_acceleration_list))
    plt.show(block=False)

def plot_sun_cone(
        sun_cone: list,
        sun_cone_in_light_index: list,
        panel_n_cone: list):
    """绘制太阳矢量圆锥"""
    sun_cone_in_light = [sun_cone[i] for i in sun_cone_in_light_index]

    fig = plt.figure()
    ax = fig.add_subplot(111, projection='3d')
    ax.set_xlabel('x')
    ax.set_ylabel('y')
    ax.set_zlabel('z')
    ax.set_title('Sun Cone in VVLH')
    for v_sun in sun_cone:
        ax.quiver(0, 0, 0, v_sun[0], v_sun[1], v_sun[2], arrow_length_ratio=0.1)

    for v_sun in sun_cone_in_light:
        ax.quiver(0, 0, 0, v_sun[0], v_sun[1], v_sun[2], color='r', arrow_length_ratio=0.1)

    for v_sun in panel_n_cone:
        ax.quiver(0, 0, 0, v_sun[0], v_sun[1], v_sun[2], color='g', arrow_length_ratio=0.1)

    ax.set_xlim(-1.5, 1.5)
    ax.set_ylim(-1.5, 1.5)
    ax.set_zlim(-1.5, 1.5)
    ax.set_aspect('equal', adjustable='box')  # 使用 'box' 以保持轴标签的比例·
    plt.show(block=False)

    angle_light = []
    angle_shadow = []
    roll_angle = []
    for index, (v_sun, v_panel) in enumerate(zip(sun_cone, panel_n_cone)):
        if index in sun_cone_in_light_index:
            angle_light.append(vector_angle(v_sun, v_panel))
        else:
            angle_shadow.append(vector_angle(v_sun, v_panel))
        roll_angle.append(np.pi/2 - vector_angle(v_panel, np.array([0, 0, -1])))

    print(f"阳照区太阳与帆板法向夹角最大值: {np.rad2deg(max(angle_light)):.2f} deg")
    print(f"阴影区太阳与帆板法向夹角最大值: {np.rad2deg(max(angle_shadow)):.2f} deg")
    print(f"滚转角最大值: {np.rad2deg(max(roll_angle)):.2f} deg")
    print(f"滚转角最小值: {np.rad2deg(min(roll_angle)):.2f} deg")

    # fig = plt.figure()
    # ax = fig.add_subplot()
    # ax.plot(np.rad2deg(angle_light))
    # ax.plot(np.rad2deg(angle_shadow))
    # ax.grid()
    # plt.show(block=False)

    return roll_angle

def plot_orbit_frame(ax: plt.Axes):
    """绘制轨道坐标系"""
    ax.quiver(-1, 0, 2, 0, color="b", scale=1, scale_units="xy")
    ax.quiver(0, 1, 0, -2, color="b", scale=1, scale_units="xy")
    ax.text(1, 0 - 0.1, "$y_O$", fontsize=10, color="b")
    ax.text(0, -1 - 0.1, "$z_O$", fontsize=10, color="b")
    ax.set_aspect('equal', adjustable='box')
    ax.set_xlim(-1.5, 1.5)
    ax.set_ylim(-1.5, 1.5)

    ax.set_xticks([])  # 删除x轴刻度
    ax.set_yticks([])  # 删除y轴刻度
    ax.grid(False)     # 删除网格

    # 绘制太阳矢量
    beta_min = 4.6 # deg
    beta_max = 19.2 # deg

    ax.quiver(0,0,
            np.cos(np.deg2rad(beta_min+90)),
            np.sin(np.deg2rad(beta_min+90)),
            color="y", scale=0.9, scale_units="xy")

    ax.text(np.cos(np.deg2rad(beta_min+90))-0.2,
            np.sin(np.deg2rad(beta_min+90))+0.2,
            "$sun$", fontsize=10, color="y")

    ax.quiver(0,0,
            np.cos(np.deg2rad(beta_max + 90)),
            np.sin(np.deg2rad(beta_max + 90)),
            color="y", scale=0.9, scale_units="xy")

def rotate_2d_vector(vector, alpha):
    """
    旋转二维向量
    Args:
        vector: 要旋转的二维向量 (x, y)
        alpha: 旋转角度（弧度制）绕z轴旋转为正
    Returns:
        旋转后的二维向量
    """
    # 定义旋转矩阵
    rotation_matrix = np.array([
        [np.cos(alpha), -np.sin(alpha)],
        [np.sin(alpha),  np.cos(alpha)]
    ])

    # 进行矩阵乘法
    rotated_vector = np.dot(rotation_matrix, vector)
    return rotated_vector

def plot_body_frame(ax: plt.Axes, roll_angle: float):
    """
    绘制卫星本体坐标系，YOZ平面
    Args:
        roll_angle: 旋转角度（弧度制）绕z轴旋转为正
    """
    gnss_install_angle = np.deg2rad(0)
    gnss_vector = np.array([np.sin(gnss_install_angle), np.cos(gnss_install_angle)])
    start_points_y = np.array([-1,0])
    start_points_z = np.array([0,1])
    end_points_y = np.array([1, 0])
    end_points_z = np.array([0, -1])

    ax.quiver(rotate_2d_vector(start_points_y, -roll_angle),
              rotate_2d_vector(start_points_z, -roll_angle),
              rotate_2d_vector(end_points_y-start_points_y, -roll_angle),
              rotate_2d_vector(end_points_z-start_points_z, -roll_angle),
              color="r", scale=1, scale_units="xy")

    end_points_y_roll = rotate_2d_vector(end_points_y, -roll_angle)
    end_points_z_roll = rotate_2d_vector(end_points_z, -roll_angle)

    ax.text(end_points_y_roll[0], end_points_z_roll[0]-0.1, "$y_B$", fontsize=10, color="r")
    ax.text(end_points_y_roll[1], end_points_z_roll[1]-0.1, "$z_B$", fontsize=10, color="r")

    # # 绘制GNSS天线方向
    # gnss_vector_roll = rotate_2d_vector(gnss_vector, -roll_angle)
    # ax.quiver(0,0,
    #           gnss_vector_roll[0],
    #           gnss_vector_roll[1],
    #           color="g", scale=0.9, scale_units="xy")

    # gnss_up_angle = vector_angle(gnss_vector_roll, np.array([0, 1]))
    # ax.text(-1, -1.2, f"GNSS angle: {np.rad2deg(gnss_up_angle):.1f} deg", fontsize=10)

    # 绘制星敏光轴方向
    # star_tracker_angle = [np.deg2rad(40), np.deg2rad(120)]
    # star_tracker_vector = []
    # star_tracker_vector.append([np.cos(star_tracker_angle[0]), np.sin(star_tracker_angle[0])]) 
    # star_tracker_vector.append([np.cos(star_tracker_angle[1]), np.sin(star_tracker_angle[1])])

    # star_tracker_vector_roll = []
    # star_tracker_vector_roll.append(rotate_2d_vector(star_tracker_vector[0], -roll_angle))
    # star_tracker_vector_roll.append(rotate_2d_vector(star_tracker_vector[1], -roll_angle))
    # ax.quiver(0,0,
    #           star_tracker_vector_roll[0][0],
    #           star_tracker_vector_roll[0][1],
    #           color="g", scale=0.9, scale_units="xy")
    # ax.quiver(0,0,
    #           star_tracker_vector_roll[1][0],
    #           star_tracker_vector_roll[1][1],
    #           color="g", scale=0.9, scale_units="xy")

    # ax.text(star_tracker_vector_roll[0][0],
    #         star_tracker_vector_roll[0][1]+0.1,
    #         "star_A", fontsize=10, color="g")
    # ax.text(star_tracker_vector_roll[1][0],
    #         star_tracker_vector_roll[1][1]+0.1,
    #         "star_B", fontsize=10, color="g")

    ax.text(-1, -1.3, f"roll angle: {np.rad2deg(roll_angle):.1f} deg", fontsize=10)



def plot_body_in_orbit_frame(sun_light_roll: float, sun_shadow_roll: float):
    """
    绘制卫星在轨道坐标系中的姿态
    """
    _, ax = plt.subplots(2, 2, figsize=(10, 8))

    ax[0, 0].set_title("sun - light")
    plot_orbit_frame(ax[0, 0])
    plot_body_frame(ax[0, 0], sun_light_roll)

    ax[0, 1].set_title("sun - shadow")
    plot_orbit_frame(ax[0, 1])
    plot_body_frame(ax[0, 1], sun_shadow_roll)

    ax[1, 0].set_title("earth - left swing")
    plot_orbit_frame(ax[1, 0])
    plot_body_frame(ax[1, 0], np.deg2rad(30))

    ax[1, 1].set_title("earth - right swing")
    plot_orbit_frame(ax[1, 1])
    plot_body_frame(ax[1, 1], np.deg2rad(-30))

    # Adjust layout to prevent overlap
    plt.tight_layout()

    # Show the plot
    plt.show(block=False)
    plt.savefig("att_in_orbit_frame.png", dpi=300)  # 保存为高分辨率图片

def check_star_tracker():
    """检查星敏光轴视场（与太阳和地球的夹角）"""
    # 星敏光轴矢量定义
    a1 = np.deg2rad(33)
    a2 = np.deg2rad(43)
    s1_b = np.array([ np.sin(a1), -np.cos(a1)*np.cos(a2), -np.cos(a1)*np.sin(a2)])
    s2_b = np.array([-np.sin(a1), -np.cos(a1)*np.cos(a2), -np.cos(a1)*np.sin(a2)])
    s_b_list = [s1_b, s2_b]

    # 检查约束对日星敏的视场
    beta = -19.2
    sun_cone = sun_cone_in_vvlh(np.deg2rad(beta))
    panel_n_cone = panel_normal_cone(sun_cone)
    r_bo_list = [calc_sun_att(panel_n) for panel_n in panel_n_cone]
    calc_star_tracker_field_of_view(s_b_list, sun_cone, r_bo_list, f'Sun Attitude, beta={beta} deg')

    # 检查对地模式星敏的视场
    # 右侧视
    r_bo_list = [euler2dcm('ZYX', [3.8, 0, -30], degrees=True) for _ in range(len(sun_cone))]
    calc_star_tracker_field_of_view(s_b_list, sun_cone, r_bo_list, f'Right Swing 30 deg, beta={beta} deg')

    # 左侧视
    r_bo_list = [euler2dcm('ZYX', [3.8, 0, 30], degrees=True) for _ in range(len(sun_cone))]
    calc_star_tracker_field_of_view(s_b_list, sun_cone, r_bo_list, f'Left Swing 30 deg, beta={beta} deg')

    # 正对地
    r_bo_list = [euler2dcm('ZYX', [0, 0, 0], degrees=True) for _ in range(len(sun_cone))]
    calc_star_tracker_field_of_view(s_b_list, sun_cone, r_bo_list, f'Earth [0,0,0], beta={beta} deg')

    # 检查轨控模式星敏视场
    # # +X推进
    r_bo_list = [euler2dcm('XYZ', [0, 90, 90], degrees=True) for _ in range(len(sun_cone))]
    calc_star_tracker_field_of_view(s_b_list, sun_cone, r_bo_list, f'Orbit control +X, beta={beta} deg')

    # # -X推进
    r_bo_list = [euler2dcm('XYZ', [0, -90, -90], degrees=True) for _ in range(len(sun_cone))]
    calc_star_tracker_field_of_view(s_b_list, sun_cone, r_bo_list, f'Orbit control -X, beta={beta} deg')

    # # -Y推进
    r_bo_list = [euler2dcm('ZYX', [0, 0, 90], degrees=True) for _ in range(len(sun_cone))]
    calc_star_tracker_field_of_view(s_b_list, sun_cone, r_bo_list, f'Orbit control -Y, beta={beta} deg')

def calc_star_tracker_field_of_view(
        s_b_list: list[np.ndarray],
        sun_cone: list, 
        r_bo_list: list[np.ndarray],
        title: str):
    """
    计算星敏光轴与太阳矢量夹角
    s_b_list: 星敏光轴列表。例：两个星敏就是两个三维矢量
    sun_cone: 太阳矢量列表
    r_bo_list: 轨道系到本体系的旋转矩阵列表（与sun_cone 一一对应）
    """

    star_tracker_num = len(s_b_list)
    sun_angle = [[] for _ in range(star_tracker_num)]
    earth_angle = [[] for _ in range(star_tracker_num)]
    earth = np.array([0, 0, 1])

    for v_sun, r_bo in zip(sun_cone, r_bo_list):
        for index, s_b in enumerate(s_b_list):
            r_ob = r_bo.transpose()
            s_o = r_ob @ s_b  # 星敏光轴在轨道系下的矢量
            sun_angle[index].append(vector_angle(v_sun, s_o))
            earth_angle[index].append(vector_angle(earth, s_o))

    alpha = np.linspace(0, 2*np.pi, sample_num)

    fig, ax = plt.subplots(1, 2, figsize=(12, 6))
    # plt.subplots_adjust(hspace=0.35)  # 增加两幅图的纵向间距

    fig.suptitle(title, fontsize=16)
    ax[0].set_title("Star Tracker Light Axis to Sun")
    for index, theta in enumerate(sun_angle):
        ax[0].plot(np.rad2deg(alpha), np.rad2deg(theta), label=f"star tracker {index+1}")
        mark_max_min(ax[0], np.rad2deg(alpha), np.rad2deg(theta))
    ax[0].set_xlabel("Phase (deg)")
    ax[0].set_ylabel("Angle (deg)")
    ax[0].grid(True)
    ax[0].legend()

    ax[1].set_title("Star Tracker Light Axis to Earth")
    for index, theta in enumerate(earth_angle):
        ax[1].plot(np.rad2deg(alpha), np.rad2deg(theta), label=f"star tracker {index+1}")
        mark_max_min(ax[1], np.rad2deg(alpha), np.rad2deg(theta))
    ax[1].set_xlabel("Phase (deg)")
    ax[1].set_ylabel("Angle (deg)")
    ax[1].grid(True)
    ax[1].legend()

    plt.show(block=False)

def mark_max_min(ax, x, y, label=None, color_max='red', color_min='blue'):
    """
    在ax上标记y的最大值和最小值
    ax: matplotlib的axes对象
    x: x数据（1维数组）
    y: y数据（1维数组）
    label: 可选，标注前缀
    color_max: 最大值点颜色
    color_min: 最小值点颜色
    """
    y = np.array(y)
    x = np.array(x)
    idx_max = np.argmax(y)
    idx_min = np.argmin(y)
    # 标记最大值
    ax.scatter(x[idx_max], y[idx_max], color=color_max, marker='o')
    txt = f"max:{y[idx_max]:.1f}" if label is None else f"{label} max:{y[idx_max]:.1f}"
    ax.annotate(txt, (x[idx_max], y[idx_max]), textcoords="offset points", xytext=(0,10), ha='center', color=color_max)
    # 标记最小值
    ax.scatter(x[idx_min], y[idx_min], color=color_min, marker='o')
    txt = f"min:{y[idx_min]:.1f}" if label is None else f"{label} min:{y[idx_min]:.1f}"
    ax.annotate(txt, (x[idx_min], y[idx_min]), textcoords="offset points", xytext=(0,-15), ha='center', color=color_min)

def main():
    """分析主函数"""
    sun_cone = sun_cone_in_vvlh(np.deg2rad(-19.2))
    sun_cone_in_light_index = sun_vector_in_light(sun_cone, SEMIMAJAR_AXIS)
    panel_n_cone = panel_normal_cone(sun_cone)

    # 计算星敏光轴与太阳矢量夹角
    # check_star_tracker()

    anlyse_sun_att(panel_n_cone)

    # energy_efficiency(
    #     sun_cone,
    #     sun_cone_in_light_index,
    #     panel_n_cone)

    # roll_angle = plot_sun_cone(
    #     sun_cone,
    #     sun_cone_in_light_index,
    #     panel_n_cone)

    # plot_body_in_orbit_frame(max(roll_angle), min(roll_angle))

    input("Press Enter to exit...")


if __name__ == "__main__":
    main()



from excel_reader import ExcelReader
import pandas as pd
import os, sys
parent_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), '../../src'))
sys.path.append(parent_dir)
from aocs_lab.utils import lib

import numpy as np
import matplotlib.pyplot as plt

def example_read_excel():
    """Example of different ways to read Excel data"""
    
    # Replace with your actual Excel file path
    file_path = r"C:\Users\Administrator\Desktop\PIESAT02_C01-20250407-20250407_XXXX.xlsx"
    
    reader = ExcelReader(file_path)
    
    # Method 1: Read all data from default sheet
    print("=== Method 1: Read all data ===")
    all_data = reader.read_sheet_data()
    for i, row in enumerate(all_data[:3]):
        print(f"Row {i+1}: {row}")
    
    # Method 2: Read specific sheet
    print("\n=== Method 2: Read specific sheet ===")
    sheets = reader.get_sheet_names()
    if sheets:
        sheet_data = reader.read_sheet_data(sheets[0])
        print(f"Sheet '{sheets[0]}' has {len(sheet_data)} rows")
    
    # Method 3: Read specific range
    print("\n=== Method 3: Read specific range ===")
    if sheets:
        range_data = reader.read_range(sheets[0], 'A1', 'C5')
        print("Range A1:C5:")
        for row in range_data:
            print(row)
    
    # Method 4: Use pandas for advanced operations
    print("\n=== Method 4: Use pandas ===")
    df = reader.read_with_pandas()
    if not df.empty:
        print(f"DataFrame info:")
        print(f"Shape: {df.shape}")
        print(f"Columns: {list(df.columns)}")
        print("\nFirst few rows:")
        print(df.head())

def read_excel():
    # Replace with your actual Excel file path
    file_path = r"PIESAT02_C01-20250407-20250407_XXXX.xlsx"
    
    reader = ExcelReader(file_path)
    
    all_data = reader.read_sheet_data()

    # print(all_data)

    timeline = []
    gnss_angles = []
    omega_norm = []
    for i, row in enumerate(all_data[1:]):
        # print(f"Row {i+1}: {row[3:7]}")
        q_BO = row[3:7]
        dcm_BO = lib.quat2dcm(q_BO)
        gnss_B = np.array([0, np.cos(np.deg2rad(60)), -np.sin(np.deg2rad(60))])
        gnss_O = dcm_BO.T @ gnss_B
        sky_vector_O = np.array([0, 0, -1])
        gnss_angles.append(lib.vector_angle(gnss_O, sky_vector_O))

        timeline.append(row[0])

        omega = np.array(row[7:10])
        omega_norm.append(np.linalg.norm(omega))


    time_data = pd.to_datetime(timeline)
    np.savetxt('gnss_angles.txt', np.rad2deg(gnss_angles), fmt='%.6f')

    fig, axs = plt.subplots(2, 1, figsize=(12, 10))
    axs[0].plot(time_data, np.rad2deg(gnss_angles) , marker='o', linestyle='-', color='b')
    axs[0].set_title('GNSS Angles Over Time')
    axs[0].set_xlabel('Time')
    axs[0].set_ylabel('Angle (degrees)')
    axs[0].grid()

    axs[1].plot(time_data, omega_norm, marker='o', linestyle='-', color='r')
    axs[1].set_title('Angular Velocity Norm Over Time')
    axs[1].set_xlabel('Time')
    axs[1].set_ylabel('Angular Velocity Norm (rad/s)')
    axs[1].grid()

    plt.tight_layout()
    plt.show()


if __name__ == "__main__":
    read_excel()



"""获取和处理 TLE 数据"""
import json
import argparse
import requests

def get_tle_json_file(norad_cat_id: int,
                      date_start: str,
                      date_end: str,
                      site_cred: dict,
                      file_path: str):
    """
    date_start and date_end should be in the format of "YYYY-MM-DD"
    """
    uri_base = "https://www.space-track.org"
    request_login = "/ajaxauth/login"
    request_controller = "/basicspacedata"
    request_action = "/query"
    request_class = f"/class/gp_history/NORAD_CAT_ID/{norad_cat_id}"
    request_class_date = f"/orderby/TLE_LINE1 ASC/EPOCH/{
        date_start}--{date_end}"
    request_class_format = "/format/json"

    with requests.Session() as session:
        resp = session.post(uri_base + request_login, data=site_cred)
        if resp.status_code != 200:
            raise ValueError("POST fail on login")

        resp = session.get(uri_base +
                           request_controller +
                           request_action +
                           request_class +
                           request_class_date +
                           request_class_format)
        if resp.status_code != 200:
            print(resp)
            raise ValueError(
                resp, "GET fail on request for Starlink satellites")

        ret_data = json.loads(resp.text)

        # Save JSON response to a file using UTF-8 encoding
        with open(file_path, 'w', encoding='utf-8') as json_file:
            json.dump(ret_data, json_file, indent=4)

        session.close()


if __name__ == "__main__":
    # 参数解析
    parser = argparse.ArgumentParser(description="TLE 数据获取与处理")
    parser.add_argument('-d', '--download',
                        action='store_true', help="下载 TLE 数据")
    parser.add_argument('-a', '--analyse',
                        action='store_true', help="分析 TLE 数据")
    args = parser.parse_args()

    cred = {'identity': '', 'password': ''}
    sat_ids = [62333, 62334, 62335, 62336]

    if args.download:
        for sat_id in sat_ids:
            get_tle_json_file(
                sat_id,
                "2024-12-24",
                "2024-12-27",
                cred,
                f"./{sat_id}.json")

    if args.analyse:
        for sat_id in sat_ids:
            with open(f"{sat_id}.json", 'r', encoding='utf-8') as file:
                data = json.load(file)

            print(f"{sat_id}: {data[-1]['SEMIMAJOR_AXIS']} km")




import numpy as np
import pprint

import os
import sys


def get_max_magnetic_disterbuance(
        mag_moment_change_matrix, 
        mag_moment_change_max, 
        magmeter_change_matrix):
    """
    计算磁力矩器对磁强计测量值的最大干扰

    Parameters
    ----------
        mag_moment_change_matrix : 
            标准磁矩变化矩阵 (3x3), 每行一组输出值，一共三组
        mag_moment_change_max : 
            标准磁矩变化最大值 (3x1)。若磁棒最大输出 25 Am^2, 则为[25, 25, 25]
        magnetic_change_matrix : 
            标准磁矩变化矩阵 (3x3), 每行一组测量值，一共三组

    Returns
    -------
        max_magmeter_change : 磁强计测量值的最大干扰量
    """

    # 磁矩与磁强计关系矩阵 (3x3)
    # magmeter_change_matrix = mag_moment_change_matrix @ k
    # [bx, by, bz] = [mx, my, mz] @ k
    k = magmeter_change_matrix @ np.linalg.inv(mag_moment_change_matrix)

    # 枚举磁力矩器最大输出立方体角点
    dir = np.array([[1, 1, 1],
                    [1, 1, -1],
                    [1, -1, 1],
                    [1, -1, -1],
                    [-1, 1, 1],
                    [-1, 1, -1],
                    [-1, -1, 1],
                    [-1, -1, -1]])
    
    m_max_list = mag_moment_change_max * dir

    b_max_list = m_max_list @ k

    # 行向量取模
    b_row_norms = np.linalg.norm(b_max_list, axis=1)

    return max(b_row_norms)


if __name__ == "__main__":
    # 标准磁矩变化矩阵
    dm = np.array([[25, 0, 0],
                   [0, 25, 0],
                   [0, 0, 25]])


    # A星 20250212 磁强计A
    b_std = np.array([[0.0000318, 0.00000831, 0.0000301],
                      [0.0000318, 0.00000831, 0.0000301],
                      [0.0000318, 0.00000831, 0.0000301]])

    # 给定标准磁矩变化矩阵，磁强计测量值
    b_output = np.array([[0.0000282, 0.0000101,  0.0000301],
                         [0.0000311, 0.00000681, 0.0000309],
                         [0.0000407, 0.00000956, 0.0000358]])


    # # B星
    # # 磁强计测量基准值
    # b_std = np.array([[-0.0547, 0.392, 0.292],
    #                   [-0.0547, 0.392, 0.292],
    #                   [-0.0547, 0.392, 0.292]])

    # # 给定标准磁矩变化矩阵，磁强计测量值
    # b_output = np.array([[-0.0722, 0.395, 0.279],
    #                      [-0.0596, 0.364, 0.292],
    #                      [-0.0533, 0.411, 0.290]])


    db_max = get_max_magnetic_disterbuance(dm, [25, 25, 25], b_output - b_std)

    print(f"磁棒对磁强计的最大扰动为 {db_max*1e4:.2e}")



"""获取和处理 TLE 数据"""
import os
import sys
import json
import argparse
import requests

parent_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), '../../src'))
sys.path.append(parent_dir)
import aocs_lab.spacetrack.tle as tle


if __name__ == "__main__":
    # 参数解析
    parser = argparse.ArgumentParser(description="TLE 数据获取与处理")
    parser.add_argument('-d', '--download',
                        action='store_true', help="下载 TLE 数据")
    parser.add_argument('-a', '--analyse',
                        action='store_true', help="分析 TLE 数据")
    args = parser.parse_args()

    cred = {'identity': 'wzhmwz@163.com', 'password': 'JXC5urv_rqr4gup3wmd'}
    sat_ids = [62333, 62334, 62335, 62336]

    if args.download:
        for sat_id in sat_ids:
            tle.get_tle_json_file(
                sat_id,
                "2024-12-24",
                "2024-12-27",
                cred,
                f"./{sat_id}.json")

    if args.analyse:
        for sat_id in sat_ids:
            file_path = f"{sat_id}.json"
            with open(file_path, 'r', encoding='utf-8') as file:
                data = json.load(file)

            print(f"{sat_id}: {data[-1]['SEMIMAJOR_AXIS']} km")


# A: 6904.259 km, 75.921 deg
# B: 6903.398 km, 85.391 deg
# C: 6903.139 km, 88.081 deg
# D: 6902.575 km, 94.432 deg



import win32com.client
import datetime

# Get reference to running STK instance
uiApplication = win32com.client.Dispatch('STK11.Application')
uiApplication.Visible = True

# Get our IAgStkObjectRoot interface
root = uiApplication.Personality2



# 选择卫星
def sat_propagate(name: str, orbit_epoch: datetime.datetime, orbit_rv_fix_m: list):
    satellite = root.CurrentScenario.Children.new(18, name)  # eSatellite

    # 设置递推类型 HPOP
    satellite.SetPropagatorType(0) # 0: ePropagatorHPOP
    print(satellite.PropagatorSupportedTypes)
    print(satellite.PropagatorType)

    # 设置 HPOP 摄动力
    forceModel = satellite.Propagator.ForceModel
    forceModel.CentralBodyGravity.File = r'C:\Program Files\AGI\STK 11\STKData\CentralBodies\Earth\WGS84_EGM96.grv'
    forceModel.CentralBodyGravity.MaxDegree = 21
    forceModel.CentralBodyGravity.MaxOrder = 21
    forceModel.Drag.Use=0
    forceModel.SolarRadiationPressure.Use=0

    # 设置递推初始时间与rv
    satellite.Propagator.InitialState.OrbitEpoch.SetExplicitTime(
        orbit_epoch.strftime('%d %b %Y %H:%M:%S.%f'))
    
    orbit_rv_fix_km = [x/1e3 for x in orbit_rv_fix_m] # 单位转换
    satellite.Propagator.InitialState.Representation.AssignCartesian(
        2, 
        orbit_rv_fix_km[0], 
        orbit_rv_fix_km[1], 
        orbit_rv_fix_km[2], 
        orbit_rv_fix_km[3], 
        orbit_rv_fix_km[4], 
        orbit_rv_fix_km[5])  # 2: eCoordinateSystemFixed

    # 递推计算
    satellite.Propagator.Propagate()

    return satellite


# Get reference to running STK instance
uiApplication = win32com.client.Dispatch('STK11.Application')
uiApplication.Visible = True

# Get our IAgStkObjectRoot interface
root = uiApplication.Personality2


GNSS_A_data = [
    1728863912,
    799616.31,
    3015870.75,
    6147789.00,
    675.43,
    -6897.96,
    3288.37]

GNSS_B_data = [
    1728863903,
    793314.56,
    3077794.00,
    6117463.00,
    693.34,
    -6864.36,
    3355.6]

GNSS_C_data = [
    1728863919,
    804203.00,
    2968066.75,
    6170752.50,
    661.47,
    -6923.44,
    3236.07
]

GNSS_D_data = [
    1728863913,
    800546.13,
    3008147.75,
    6151491.00,
    673.33,
    -6901.90,
    3280.39,
]

satellite_A = sat_propagate('A', datetime.datetime.fromtimestamp(GNSS_A_data[0] - 8*3600), GNSS_A_data[1:])
satellite_B = sat_propagate('B', datetime.datetime.fromtimestamp(GNSS_B_data[0] - 8*3600), GNSS_B_data[1:])
satellite_C = sat_propagate('C', datetime.datetime.fromtimestamp(GNSS_C_data[0] - 8*3600), GNSS_C_data[1:])
satellite_D = sat_propagate('D', datetime.datetime.fromtimestamp(GNSS_D_data[0] - 8*3600), GNSS_D_data[1:])




from dataclasses import dataclass
from datetime import datetime, timedelta
import numpy as np
from astropy.time import Time
from astropy.coordinates import get_sun
from astropy import units as u
from .utils.constants import J2, EARTH_EQUATORIAL_RADIUS as R, GM_EARTH as MU


@dataclass
class Orbit:
    """
    轨道类，包含轨道参数和相关计算方法。

    Attributes:
        a: 轨道半长轴 (m)
        e: 轨道偏心率
        i: 轨道倾角 (rad)
        raan: 升交点赤经 (rad)
        arg_perigee: 近地点幅角 (rad)
        true_anomaly: 真近点角 (rad)
        time: 近地点时间 (datetime)
    """
    a: float
    e: float
    i: float
    raan: float
    arg_perigee: float
    true_anomaly: float
    time: datetime


def orbit_period(a: float) -> float:
    """
    根据轨道半长轴计算轨道周期

    Args:
        a: 轨道半长轴 (m)

    Returns:
        orbit_period: 轨道周期 (s)
    """
    # Kepler's Third laws of planetary motion
    return np.sqrt(4*np.pi**2 * a**3 / MU)


def raan_dot(a, e, i):
    """
    计算升交点赤经变化率
    参考 Curtis2020 2.45

    Args:
        a: 轨道半长轴 (m)
        e: 轨道偏心率
        i: 轨道倾角 (rad)

    Returns:
        raan_dot: 升交点赤经变化率 (rad/s)
    """

    raan_dot = (-1.5 * J2 * np.sqrt(MU) * R**2 * np.cos(i) /
                a**(7/2) * (1 - e**2)**2)

    return raan_dot


def orbit_normal(i: float, raan: float) -> np.ndarray:
    """
    计算轨道面法向量
    参考 Curtis2020 4.49

    Args:
        i: 轨道倾角 (rad)
        raan: 升交点赤经 (rad)

    Returns:
        n: 轨道面法向量 (np.ndarray)，单位矢量
    """

    n = np.array([
        np.sin(i) * np.sin(raan),
        -np.sin(i) * np.cos(raan),
        np.cos(i)
    ])

    return n


def beta_angle(h_hat: np.ndarray, s_hat: np.ndarray) -> float:
    """
    计算beta角

    注意，beta角定义为轨道面法向量与太阳矢量的夹角的补角，
    即 beta = 90° - angle(h_hat, s_hat)

    Args:
        h_hat: 轨道面法向量 (np.ndarray)，单位矢量
        s_hat: 太阳矢量 (np.ndarray)，单位矢量

    Returns:
        beta: beta角 (rad)
    """
    return np.arcsin(np.clip(np.dot(h_hat, s_hat), -1.0, 1.0))


def sunlight_time_per_orbit(a: float, beta: float) -> float:
    """
    计算卫星每轨道在阳光照射下的时间

    参考 Vallado 4ed, page 305

    Args:
        a: 轨道半长轴 (m)
        beta: beta角 (rad)

    Returns:
        t_sunlight: 每轨道阳光照射时间 (s)
    """

    # 日蚀半角

    if np.abs(np.sin(beta)) > R/a:
        phi = 0
    else:
        phi = np.acos(np.sqrt(1-(R/a)**2) / np.cos(beta))

    theta_eclipse = 2 * phi

    t_orbit = orbit_period(a)
    t_eclipse = theta_eclipse / (2*np.pi) * t_orbit
    t_sunlight = t_orbit - t_eclipse

    return t_sunlight


def beta_angle_and_sunlight_per_year(orbit_init: Orbit, year: int) -> list[tuple[datetime, float, float]]:
    """
    计算beta角和每轨道阳光照射时间

    Args:
        orbit_init: 初始轨道参数，包含轨道元素和近地点时间

    Returns:
        list of tuples: 每个元组包含(时间, beta角, 每轨道阳光照射时间)
    """

    t_orbit = Time(orbit_init.time, scale='ut1')
    start = datetime(year, 1, 1)
    end = datetime(year, 12, 31)
    days = (end - start).days + 1

    results = []
    for i in range(days):
        t = start + timedelta(days=i)

        # 计算太阳矢量
        s = get_sun(Time(t))
        s_vec = s.cartesian.xyz.value
        s_hat = s_vec / np.linalg.norm(s_vec)

        # todo：此处未考虑真赤道与惯性系xy面的角度，需要改进
        raan_dot_value = raan_dot(orbit_init.a, orbit_init.e, orbit_init.i)
        raan = (orbit_init.raan + raan_dot_value
                * (Time(t).jd - t_orbit.jd) * 86400)

        beta = beta_angle(
            h_hat=orbit_normal(orbit_init.i, raan),
            s_hat=s_hat)

        sunlight_time = sunlight_time_per_orbit(orbit_init.a, beta)

        results.append((
            t,
            beta,
            sunlight_time))

    return results


def plot_beta_and_sunlight(results: list[tuple[datetime, float, float]]):
    """
    绘制beta角和每轨道阳光照射时间的曲线

    Args:
        results: beta_angle_and_sunlight_per_year 的输出结果

    Returns:
        fig: matplotlib figure object
    """
    import matplotlib.pyplot as plt

    times = [r[0] for r in results]
    betas = [np.rad2deg(r[1]) for r in results]
    sunlight_times = [r[2] / 60 for r in results]  # Convert to minutes

    fig = plt.figure(figsize=(10, 8))

    plt.subplot(2, 1, 1)
    plt.plot(times, betas, label='Beta Angle')

    # Mark max/min for Beta
    max_beta = np.max(betas)
    min_beta = np.min(betas)
    max_beta_idx = np.argmax(betas)
    min_beta_idx = np.argmin(betas)

    plt.scatter([times[max_beta_idx]], [max_beta], color='red', zorder=5)
    plt.text(times[max_beta_idx], max_beta,
             f'Max: {max_beta:.2f}, {times[max_beta_idx].strftime("%Y-%m-%d")}', verticalalignment='bottom')

    plt.scatter([times[min_beta_idx]], [min_beta], color='green', zorder=5)
    plt.text(times[min_beta_idx], min_beta,
             f'Min: {min_beta:.2f}, {times[min_beta_idx].strftime("%Y-%m-%d")}', verticalalignment='top')

    plt.ylabel('Beta Angle (deg)')
    plt.title('Beta Angle vs Time')
    plt.grid(True)

    plt.subplot(2, 1, 2)
    plt.plot(times, sunlight_times, label='Sunlight Time', color='orange')

    # Mark max/min for Sunlight Time
    max_sun = np.max(sunlight_times)
    min_sun = np.min(sunlight_times)
    max_sun_idx = np.argmax(sunlight_times)
    min_sun_idx = np.argmin(sunlight_times)

    plt.scatter([times[max_sun_idx]], [max_sun], color='red', zorder=5)
    plt.text(times[max_sun_idx], max_sun,
             f'Max: {max_sun:.2f}, {times[max_sun_idx].strftime("%Y-%m-%d")}', verticalalignment='bottom')

    plt.scatter([times[min_sun_idx]], [min_sun], color='green', zorder=5)
    plt.text(times[min_sun_idx], min_sun,
             f'Min: {min_sun:.2f}, {times[min_sun_idx].strftime("%Y-%m-%d")}', verticalalignment='top')

    plt.ylabel('Sunlight Time (min)')
    plt.title('Sunlight Time vs Time')
    plt.xlabel('Time')
    plt.grid(True)

    plt.tight_layout()
    return fig


def raan_from_ltdn(epoch: datetime, ltdn_hours: float) -> float:
    """
    由降交点地方时计算升交点赤经 Ω

    Args:
        epoch: 计算时刻 (UTC 时间)
        ltdn_hours: 降交点地方时, 0~24 (小时)

    Returns:
        升交点赤经 RAAN Ω, 单位 rad, 0~2π
    """
    t = Time(epoch, scale="utc")

    # 1) LTDN -> LTAN
    ltan = (ltdn_hours + 12.0) % 24.0

    # 2) 太阳视赤经（apparent）
    sun = get_sun(t)  # GCRS
    alpha_sun = sun.ra.to(u.deg).value  # deg

    # 3) Ω = α_sun + 15°*(LTAN-12h)
    omega = alpha_sun + 15.0 * (ltan - 12.0)
    omega = omega % 360.0
    return np.deg2rad(omega)


if __name__ == "__main__":
    # orbit = Orbit(
    #     a=6871030,
    #     e=0,
    #     i=np.deg2rad(97.3747),
    #     raan=np.deg2rad(79.8202),
    #     arg_perigee=0,
    #     true_anomaly=0,
    #     time=datetime(2025, 1, 1)
    # )

    # res = beta_angle_and_sunlight_per_year(orbit, 2025)

    # fig = plot_beta_and_sunlight(res)
    # fig.savefig("beta_sunlight_2025.png", dpi=300)

    # raan = raan_from_ltdn(datetime(2025, 12, 16, 0, 0, 0), 10.5)
    # print(raan)



from datetime import datetime, timedelta
import time
import sys

def beijing_to_utc_time_str(beijing_time_str: str):
    # 将字符串转换为 datetime 对象
    beijing_time = datetime.strptime(beijing_time_str, '%Y-%m-%dT%H:%M:%S')

    # 北京时间转为 UTC 时间，减去8小时
    utc_time = beijing_time - timedelta(hours=8)

    # 转换为字符串表示
    utc_time_str = utc_time.strftime('%Y-%m-%dT%H:%M:%SZ')

    return utc_time_str

def print_percentage(current, total, start_time):
    """计算进度与时间"""
    elapsed_time = time.time() - start_time

    progress = current / total

    if current > 0:
        estimated_total_time = elapsed_time / progress
        remaining_time = estimated_total_time - elapsed_time
    else:
        estimated_total_time = 0
        remaining_time = 0

    percent = (current / total) * 100
    sys.stdout.write(f"\rProgress: {percent:.2f}% | Elapsed: {
        elapsed_time/60:.1f} min | Remaining: {remaining_time/60:.1f} min")
    # sys.stdout.flush()


def gmst_time(jd_ut1, longitude_deg):
    # 当地平太阳时 计算 当地赤经（right ascension）
    # ALGORITHM 15 (Vallado, D.A. and McClain, W.D. (2013) Fundamentals of Astrodynamics and Applications. Fourth Edition.)
    J2000_jd  = 2_451_545 # January 1, 2000 12:00:00.000
    jd_century = 36_525
    t_ut1 = (jd_ut1 - J2000_jd) / jd_century
    theta_gmst_second = (67_310.548_41 
                + (876_600 * 3600 + 8_640_184.812_866) * t_ut1
                + 0.093_104 * t_ut1**2
                - 6.2e-6 * t_ut1**3)
    
    theta_gmst_second = theta_gmst_second % 86400
    theta_gmst_hour = theta_gmst_second / 3600
    theta_gmst_deg = theta_gmst_hour * 15

    theta_lst_deg = (theta_gmst_deg + longitude_deg) % 360
    
    return theta_gmst_deg, theta_lst_deg



if __name__ == "__main__":
    from astropy.time import Time

    # 如果升交点地方时是22:30:00，则22:30:00对应的升交点赤经可计算如下
    t = Time('2026-01-01T22:30:00', format='isot', scale='ut1')
    theta_gmst_deg, theta_lst_deg = gmst_time(t.jd, 0)
    print(theta_gmst_deg)

    # 利用 astropy 计算当地恒星时
    t = Time('2026-01-01T22:30:00', format='isot', scale='ut1', location=('0d', '0d'))
    longitude_deg = t.sidereal_time('mean').deg
    print(longitude_deg)

    # 比较计算方法偏差
    d_hour = (theta_lst_deg - longitude_deg) / 15
    d_sec = d_hour * 3600
    print(d_sec)

